/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * DiversityNavigator.java
 *
 * Created on February 1, 2002, 2:13 PM
 */

package net.metadiversity.diversity.navigator;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
// import javax.swing.border.*;

import net.metadiversity.diversity.navigator.connection.*;
import net.metadiversity.diversity.navigator.project.*;
import net.metadiversity.diversity.navigator.about.*;
import net.metadiversity.diversity.navigator.table.*;
import net.metadiversity.diversity.navigator.util.*;
import net.metadiversity.diversity.navigator.gui.DnMenuItem;
import net.metadiversity.diversity.navigator.gui.DnOptionPane;
// import net.metadiversity.diversity.navigator.collection.*;
import net.metadiversity.diversity.navigator.gui.HtmlViewer;

import org.apache.log4j.Logger;
// import org.apache.log4j.BasicConfigurator;
/**
 *
 * @version Dieter Neubacher <B>dneu@nexgo.de</B>
 */
public class DiversityNavigator extends JFrame implements ActionListener
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 1L;

    static Logger logger = Logger.getLogger( DiversityNavigator.class.getName() );

    // Font hint
    final static int SYSTEM_FONT_HINT  = 0;
    final static int LARGE_FONT_HINT   = 1;
    final static int MIXED_FONT_HINT   = 2;
    final static int FIXED_FONT_HINT   = 3;
    final static int SMALL_FONT_HINT   = 4;
                    
    private javax.swing.Box         StatusBar;
    private javax.swing.JLabel      statConnection;
    private javax.swing.JLabel      statQuery;
    private javax.swing.JDesktopPane desktop;
    
    private javax.swing.JTextArea messageOutputArea;    
    private javax.swing.JPanel    logOutputArea;    
    private String iconPath;    
    
    JMenu   mnTable, mnQuery, mnMenuExtras, mnMenuConfig, mnMenuWindow, mnMenuHelp;
    
    JButton cmdExportTab, cmdExportXml, cmdImportTab, cmdImportXml, cmdSaveTable;
    JButton cmdOpenSQL, cmdSaveSQL, cmdExecuteSQL;

    net.metadiversity.diversity.navigator.util.LogViewer logOutput;
    
    /**
    * @param args the command line arguments
    */
    public static void main( String args[] ) 
    {
        new net.metadiversity.diversity.navigator.util.PreInstall();
        // Set up a simple configuration that logs on the console.        
        String path = System.getProperty( "user.home" );
        org.apache.log4j.xml.DOMConfigurator.configure( path + "/.diversity_navigator/diversity_navigator.log.xml" );
        
        java.util.Locale.setDefault( java.util.Locale.US );

        
        // init Projects and Connections
        
        new net.metadiversity.diversity.navigator.connection.ConnectionsManager();
        new net.metadiversity.diversity.navigator.project.ProjectManager();
        // setup look and feel
        boolean usePlasticLookAndFeel = true;
        try
        {
            if( usePlasticLookAndFeel )
            {
                // Class plaf = Class.forName( "com.jgoodies.plaf.plastic.Plastic3DLookAndFeel" );                                
                // ??? ((com.jgoodies.plaf.plastic.Plastic3DLookAndFeel) plaf ).setMyCurrentTheme( Class.forName( "com.jgoodies.plaf.plastic.theme.SkyBlue" ) );
                boolean setFontHint = true;
                
              
                if( ! setFontHint )
                {
                    javax.swing.UIManager.setLookAndFeel( "com.jgoodies.plaf.plastic.Plastic3DLookAndFeel" );
                }
                else
                {                    
                    System.setProperty( com.jgoodies.looks.Options.MENU_FONT_KEY,        "LucidaBright-BOLD-20"  );
                    // System.setProperty( com.jgoodies.plaf.Options.MENU_FONT_KEY,        "dialog-BOLD-20"  );
                    System.setProperty( com.jgoodies.looks.Options.CONTROL_FONT_KEY,     "Arial-ITALIC-30" );
                    /*
                    Hint    Menu 96dpi  Control 96dpi   Menu 120dpi Control 120dpi
                    System  11 pt       11 pt           14 pt       14 pt
                    Large   12 pt       12 pt           14 pt       14 pt
                    Mixed   11 pt       11 pt           14 pt       12 pt
                    Fixed   12 pt       12 pt           12 pt       12 pt
                    Small   11 pt       11 pt           12 pt       12 pt
                     */
                    
                    int fontType = LARGE_FONT_HINT;
                    // com.jgoodies.plaf.plastic.PlasticLookAndFeel plaf = new com.jgoodies.plaf.plastic.PlasticLookAndFeel();
                    com.jgoodies.looks.plastic.Plastic3DLookAndFeel plaf = new com.jgoodies.looks.plastic.Plastic3DLookAndFeel();
                    switch( fontType )
                    {
                        case SYSTEM_FONT_HINT:    
                            plaf.setFontSizeHints( com.jgoodies.looks.FontSizeHints.SYSTEM );
                            break;
                        case LARGE_FONT_HINT:     
                            plaf.setFontSizeHints( com.jgoodies.looks.FontSizeHints.LARGE );
                            break;
                        case MIXED_FONT_HINT:     
                            plaf.setFontSizeHints( com.jgoodies.looks.FontSizeHints.MIXED );
                            break;
                        case FIXED_FONT_HINT:     
                            plaf.setFontSizeHints( com.jgoodies.looks.FontSizeHints.FIXED );
                            break;
                        case SMALL_FONT_HINT:     
                            plaf.setFontSizeHints( com.jgoodies.looks.FontSizeHints.SMALL );
                            break;
                    }
                    // User defined Font-Size
                    // plaf.setFontSizeHints( new com.jgoodies.plaf.FontSizeHints( 14, 14, 16, 16 ) );
                    
                    // com.jgoodies.plaf.Options.setUseSystemFonts( true );
                    // ((com.jgoodies.plaf.plastic.PlasticLookAndFeel) UIManager.getLookAndFeel()).setFontSizeHints( com.jgoodies.plaf.FontSizeHints.LARGE );
                    plaf.setMyCurrentTheme( new com.jgoodies.looks.plastic.theme.SkyBlue() );
                    UIManager.setLookAndFeel( plaf );                  

                    // Show GUI LAF values
                    boolean showUIDefaults = false;
                    if( showUIDefaults )
                    {
                        javax.swing.UIDefaults uid = UIManager.getDefaults();                                        
                        for( java.util.Enumeration e = uid.keys(); e.hasMoreElements(); ) 
                        {
                            String key = (String) e.nextElement();
                            
                            if( key.endsWith( ".font" ) )
                            {
                                System.out.print( key );
                                for( int p = key.length(); p < 40; p++ )
                                {
                                    System.out.print( ' ' );
                                }
                                System.out.println( key + " " + uid.get( key ) );
                            }
                        }
                    }
                }
            }
        }        
        catch( Exception ex )
        {
            logger.info( "Can't setup LookAndFeel" );
            logger.info( "Class: com.jgoodies.looks.plastic.Plastic3DLookAndFeel not found." );
        }
        // ??? REMOVE IT        
        // public static DiversityNavigatorProperties diversityNavigatorProperties;        
        // diversityNavigatorProperties = new DiversityNavigatorProperties();
        new DiversityNavigatorProperties();
        DiversityNavigator navigator = new DiversityNavigator();
        
        // Test Function for delopment only
        // navigator.test();
    }
    // Test Function for delopment only
    private void test()
    {
        try
        {
            DnDbProjectRoot projectRoot = new DnDbProjectRoot();
            DnDbProject     project     = new DnDbProject( "project" );
            DnDbGroup       group       = new DnDbGroup( "group" );
            DnDatabase      database    = new DnDatabase( "Checklists" );
            group.addDnDatabase( database );
            project.addDnGroup( group );
            projectRoot.addDnProject( project );
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
    /** Creates a new instance of DiversityNavigator */
    public DiversityNavigator() 
    {
    	super( "Diversity Navigator " + DiversityNavigatorProperties.getDiversityNavigatorVersion() );
        // Set default frame for DiversityNavigator messages  
        DnOptionPane.setDefaultParentComponent( this.getContentPane() );
    	
        logOutputArea = new javax.swing.JPanel();
        logOutputArea.setLayout( new BorderLayout() );
        
        logOutput = new LogViewer();
        logOutput.setThreshold( org.apache.log4j.Level.INFO );
        // deprecated logOutput.setThreshold( org.apache.log4j.Priority.INFO );
        logOutputArea.add( logOutput.getLogListPanel(), BorderLayout.CENTER );
        
        if( logger.isInfoEnabled() )
        {    
            logger.info( "Entering DiversityNavigator application" );
            logger.info( "Start logging for diversity-navigator" );
        }                    
        try 
        {
            iconPath = DiversityNavigatorProperties.getIconPath();        
            logger.info( "IconPath: " + iconPath );  
            if( logger.isInfoEnabled() )
            {    
                // Show all System properies
                java.util.Properties sysprops = System.getProperties();
                logger.info( "Systeninfos: " + sysprops.toString() );  
                sysprops.store( System.out, "Systeninfos" );
                logger.info( "java.vendor:      " + System.getProperty( "java.vendor" ));
                logger.info( "java.version:     " + System.getProperty( "java.version" ));
                logger.info( "java.runtime.name:" + System.getProperty( "java.runtime.name" ));
                logger.info( "os.name:          " + System.getProperty( "os.name" ));
                logger.info( "os.version:       " + System.getProperty( "os.version" ));
                logger.info( "os.arch:          " + System.getProperty( "os.arch" ));
                logger.info( "user.dir:         " + System.getProperty( "user.dir" ));
                logger.info( "user.home:        " + System.getProperty( "user.home" ));
                logger.info( "java.home:        " + System.getProperty( "java.home" ));
                logger.info( "java.class.path:  " + System.getProperty( "java.class.path" ));
            }
            setIconImage( DiversityNavigatorProperties.getDiversityIcon().getImage() ); 
        } 
        catch ( java.io.IOException ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }    
	Dimension D = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds( 50, 50, (int)(D.getWidth() - 101), (int)(D.getHeight() - 101));

	setDefaultCloseOperation( WindowConstants.DISPOSE_ON_CLOSE );
                
	addWindowListener( new java.awt.event.WindowAdapter() 
	{
            public void windowClosed(java.awt.event.WindowEvent e) 
            {
                dispose();
		System.exit( 0 );
            }
	});
        //-------------------------------------------------------------
        // horizontal Split [ DataBase/Project ] [ workspace ]
        //-------------------------------------------------------------
	JSplitPane hsp = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT  );
	JSplitPane vsp = new JSplitPane( JSplitPane.VERTICAL_SPLIT    );
                
	desktop = new JDesktopPane();

	getContentPane().setLayout( new BorderLayout() );
	getContentPane().add( hsp, BorderLayout.CENTER );
        
        //---------------------------------
        // output TabPane ( right bottom )
        //---------------------------------
        if( logger.isInfoEnabled() ) logger.info( "Build Messagees/Error/Debug" );
        
        javax.swing.JTabbedPane outputTabSheets = new javax.swing.JTabbedPane();
        
        messageOutputArea   = new javax.swing.JTextArea();
	
        javax.swing.JPanel desktopAndOutput = new javax.swing.JPanel();      
        javax.swing.JPanel outputPanel      = new javax.swing.JPanel();      
        
	desktopAndOutput.setLayout( new BorderLayout() );
	outputPanel.setLayout(      new BorderLayout() );
        
        javax.swing.JScrollPane spMessage = new JScrollPane( messageOutputArea );

        outputTabSheets.add( "Messages", spMessage );
        outputTabSheets.add( "Log",      logOutputArea   );
        outputPanel.add( outputTabSheets, BorderLayout.CENTER );
      
	vsp.add( desktop,           JSplitPane.TOP    );
	vsp.add( outputPanel,       JSplitPane.BOTTOM );
        vsp.setOneTouchExpandable( true );
        vsp.setDividerLocation( 400 );
        // vsp.setDividerLocation( 0.5 );
        //vsp.setAlignmentY( (float) 0.7 );
        desktopAndOutput.add( vsp,  BorderLayout.CENTER );
        
        desktopAndOutput.setVisible( true );
        hsp.add( desktopAndOutput,  JSplitPane.RIGHT ); 

        //--------------------------------------------------------------
	// Connections/Project Pane (left side) of Desktop
        //--------------------------------------------------------------
        if( logger.isInfoEnabled() ) logger.info( "Build Connections/Projects" );
        
	final ConnectionsPane connections = new ConnectionsPane();        
	final ProjectPane     projects    = new ProjectPane( this );        
                
        javax.swing.JTabbedPane tabSheets = new javax.swing.JTabbedPane();

        connections.setMainWindow( this );                                     
        projects.setMainWindow( this );                                     
        
        tabSheets.add( "Databases", connections );        
        tabSheets.add( "Projects",  projects    );        
        // tabSheets.setPreferredSize( new Dimension( 200,  600 ) );
        
        tabSheets.setVisible( true );
        
        hsp.add( tabSheets, JSplitPane.LEFT );
        hsp.setOneTouchExpandable( true );
        hsp.setDividerLocation( 200 );
        // hsp.setDividerLocation( 0.9 );
        // hsp.setAlignmentX( (float) 0.9  );

        //--------------------------------------------------------------
	// Menu Bar
        //--------------------------------------------------------------
        if( logger.isInfoEnabled() ) logger.info( "Build menu" );
        
	JMenuBar menu               = new JMenuBar();
	JMenu fileMenu              = new JMenu( "File" );
	JMenu mnConnections         = new JMenu( "Connections" );
        
        fileMenu.setMnemonic( KeyEvent.VK_F );
        mnConnections.setMnemonic( KeyEvent.VK_C );
                
        mnConnections.setIcon( DiversityNavigatorProperties.getMenuIconConnection() );
        
	DnMenuItem mnAddConnection       = new DnMenuItem( "file.con.add"   );        
	DnMenuItem mnEditConnection      = new DnMenuItem( "file.con.edit"  );        
	DnMenuItem mnDeleteConnection    = new DnMenuItem( "file.con.del"   );
        
        //----------------------------------------------------------------------
	mnAddConnection.addActionListener( new ActionListener()
	{
            public void actionPerformed(ActionEvent e) 
            {
		connections.addConnection();
            }
	} );        
        //----------------------------------------------------------------------
	mnEditConnection.addActionListener( new ActionListener()
	{
            public void actionPerformed(ActionEvent e) 
            {
		connections.editConnection();
            }
	} );
        //----------------------------------------------------------------------
	mnDeleteConnection.addActionListener( new ActionListener()
	{
            public void actionPerformed(ActionEvent e) 
            {
                connections.deleteConnection();
            }
        } );
        DnMenuItem mnExit = new DnMenuItem( "file.exit" );
	mnExit.addActionListener( new ActionListener()
	{
            public void actionPerformed(ActionEvent e) 
            {
                dispose();                
		System.exit(0);
            }
	} );
        
	mnConnections.add( mnAddConnection  );
	mnConnections.add( mnEditConnection );
	mnConnections.add( mnDeleteConnection );
        
	fileMenu.add( mnConnections );
	fileMenu.add( new JSeparator() );
	fileMenu.add( mnExit );

        //----------------------------------------------------------------------
        //  Menu Table
        //----------------------------------------------------------------------
	mnTable = new JMenu( "Table" );
        mnTable.setMnemonic( KeyEvent.VK_T );
        //----------------------------------------------------------------------
	DnMenuItem mnExportTabTable = new DnMenuItem( "table.export_file" );
	mnExportTabTable.addActionListener( new ActionListener()
	{
            public void actionPerformed( ActionEvent e ) 
            {
		exportTab();
            }
	} );
        //----------------------------------------------------------------------
	DnMenuItem mnExportXmlTable = new DnMenuItem( "table.export_xml" );
	mnExportXmlTable.addActionListener( new ActionListener()
	{
            public void actionPerformed( ActionEvent e ) 
            {
		exportXml();
            }
	} );
        //----------------------------------------------------------------------        
	DnMenuItem mnImportTabTable = new DnMenuItem( "table.import_file" );        
	mnImportTabTable.addActionListener( new ActionListener()
	{
            public void actionPerformed(ActionEvent e) 
            {
                importTab();
            }
	} );
        //----------------------------------------------------------------------        
	DnMenuItem mnImportXmlTable = new DnMenuItem( "table.import_xml" );
	mnImportXmlTable.addActionListener( new ActionListener()
	{
            public void actionPerformed(ActionEvent e) 
            {
                importXml();
            }
	} );
        //----------------------------------------------------------------------
	DnMenuItem mnSaveTable = new DnMenuItem( "table.save" );
	mnSaveTable.setIcon( DiversityNavigatorProperties.getSaveIcon() );
                
	mnSaveTable.addActionListener( new ActionListener()
	{
            public void actionPerformed(ActionEvent e) 
            {
		if (currentTable() != null) currentTable().save();
            }
        } );
        // add entries to Menu--------------------------------------------------
	mnTable.add( mnExportXmlTable );
	mnTable.add( mnImportXmlTable );
	mnTable.add( mnExportTabTable );
	mnTable.add( mnImportTabTable );
	mnTable.add( new JSeparator() );
	mnTable.add( mnSaveTable );
        //----------------------------------------------------------------------
        //  Query Menu
        //----------------------------------------------------------------------
	mnQuery = new JMenu( "Query" );
        mnQuery.setMnemonic( KeyEvent.VK_Q );
            
	DnMenuItem mnOpenSQL = new DnMenuItem( "query.open" );
	mnOpenSQL.addActionListener( new ActionListener()
	{
            public void actionPerformed(ActionEvent e) 
            {
		if (currentQuery() != null) currentQuery().openSQL();
            }
	} );
        
	DnMenuItem mnSaveSQL = new DnMenuItem( "query.save" );
	mnSaveSQL.addActionListener( new ActionListener()
	{
            public void actionPerformed(ActionEvent e) 
            {
                if (currentQuery() != null) currentQuery().saveSQL();
            }
	} );
        
	DnMenuItem mnExecuteSQL = new DnMenuItem( "query.execute" );
	mnExecuteSQL.addActionListener( new ActionListener()
	{
            public void actionPerformed(ActionEvent e) 
            {
		if (currentQuery() != null) currentQuery().executeQuery();
            }
	} );
	mnQuery.add( mnOpenSQL );
	mnQuery.add( mnSaveSQL );
	mnQuery.add( new JSeparator() );
	mnQuery.add( mnExecuteSQL );

	menu.add( fileMenu );
	menu.add( mnTable );
	menu.add( mnQuery );

        //----------------------------------------------------------------------
        // MENU Extras
        //----------------------------------------------------------------------
        if( logger.isInfoEnabled() ) logger.info( "Build Extras menu" );
        try
        {
            mnMenuExtras = new JMenu( "Procedures" );
            mnMenuExtras.setMnemonic( KeyEvent.VK_P );
            
            DnMenuItem mnExtrasDataTransfer     = new DnMenuItem( "extras.data_transfer"   );
            DnMenuItem mnExtrasDataValidation   = new DnMenuItem( "extras.data_validation" );
            DnMenuItem mnExtrasDataExtraction   = new DnMenuItem( "extras.data_extraction" );
            DnMenuItem mnExtrasDataTransform    = new DnMenuItem( "extras.data_transform"  );
            DnMenuItem mnExtrasDataConversion   = new DnMenuItem( "extras.data_conversion" );
            
            mnMenuExtras.add( mnExtrasDataValidation );
            mnMenuExtras.add( mnExtrasDataExtraction );
            mnMenuExtras.add( mnExtrasDataConversion );
            mnMenuExtras.add( mnExtrasDataTransform  );
            mnMenuExtras.add( mnExtrasDataTransfer   );
            
            menu.add( mnMenuExtras );
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        //----------------------------------------------------------------------
        // MENU Configuration
        //----------------------------------------------------------------------
        if( logger.isInfoEnabled() ) logger.info( "Build Configuration menu" );
        try
        {
            mnMenuConfig = new JMenu( "Configuration" );
            mnMenuConfig.setMnemonic( KeyEvent.VK_C );
            
            DnMenuItem mnConfigGlobalSettings   = new DnMenuItem( "config.global" );
            DnMenuItem mnConfigConnectivity     = new DnMenuItem( "config.connectivity" );
            DnMenuItem mnConfigSecurity         = new DnMenuItem( "config.security" );
            
            
            DnMenuItem mnFontSizeSystem         = new DnMenuItem( "font.system" );
            DnMenuItem mnFontSizeLarge          = new DnMenuItem( "font.large" );
            DnMenuItem mnFontSizeMixed          = new DnMenuItem( "font.mixed" );            
            DnMenuItem mnFontSizeFixed          = new DnMenuItem( "font.fixed" );
            DnMenuItem mnFontSizeSmall          = new DnMenuItem( "font.small" );
            
            mnFontSizeSystem.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e) { setFontType( SYSTEM_FONT_HINT ); }
            } );
            mnFontSizeLarge.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e) { setFontType( LARGE_FONT_HINT ); }
            } );
            mnFontSizeMixed.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e) { setFontType( MIXED_FONT_HINT ); }
            } );
            mnFontSizeFixed.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e) { setFontType( FIXED_FONT_HINT ); }
            } );
            mnFontSizeSmall.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e) { setFontType( SMALL_FONT_HINT ); }
            } );

            mnMenuConfig.add( mnConfigGlobalSettings  );
            mnMenuConfig.add( mnConfigConnectivity    );
            mnMenuConfig.add( mnConfigSecurity        );
            
            mnMenuConfig.add( mnFontSizeSystem );
            mnMenuConfig.add( mnFontSizeLarge );
            mnMenuConfig.add( mnFontSizeMixed );
            mnMenuConfig.add( mnFontSizeFixed );
            mnMenuConfig.add( mnFontSizeSmall );
            
            menu.add( mnMenuConfig );
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        //----------------------------------------------------------------------
        // Window menu
        //----------------------------------------------------------------------
        if( logger.isInfoEnabled() ) logger.info( "Build window menu" );
        try
        {
            mnMenuWindow = new JMenu( "Windows/Frames" );
            mnMenuWindow.setMnemonic( KeyEvent.VK_W );
            
            DnMenuItem mnWindowHorizontal   = new DnMenuItem( "window.horizontal" );
            //  Order Internal Frames horizontal
            mnWindowHorizontal.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e) 
                {
                    JInternalFrame [] frames = desktop.getAllFrames();
                    int xPos = 0;
                    int yPos = 0;
                    int width  = desktop.getWidth() / frames.length;
                    int height = desktop.getHeight();
                    int frameDistanceX = width;
                    int frameDistanceY = 0;
                    for( int i = 0; i < frames.length; i++ )
                    {
                        if( ! frames[ i ].isIcon() )
                        {
                            try
                            {
                                frames[ i ].setMaximum( false );
                                frames[ i ].reshape( xPos, yPos,  width, height );
                                xPos += frameDistanceX;
                                yPos += frameDistanceY;
                                
                                if( xPos + width  > desktop.getWidth()  ) xPos = 0;
                                if( yPos + height > desktop.getHeight() ) yPos = 0;
                            }
                            catch( java.beans.PropertyVetoException ex )
                            {
                                logger.error( ex.getMessage(), ex.fillInStackTrace() );
                            }
                        }
                    }
                }
            } );            
            DnMenuItem mnWindowVertical     = new DnMenuItem( "window.vertical" );
            //  Order Internal Frames vertical
            mnWindowVertical.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e) 
                {
                    JInternalFrame [] frames = desktop.getAllFrames();
                    int xPos = 0;
                    int yPos = 0;
                    int width  = desktop.getWidth();
                    int height = desktop.getHeight() / frames.length;
                    int frameDistanceX = 0;
                    int frameDistanceY = height;
                    for( int i = 0; i < frames.length; i++ )
                    {
                        if( ! frames[ i ].isIcon() )
                        {
                            try
                            {
                                frames[ i ].setMaximum( false );
                                frames[ i ].reshape( xPos, yPos,  width, height );
                                xPos += frameDistanceX;
                                yPos += frameDistanceY;
                                
                                if( xPos + width  > desktop.getWidth()  ) xPos = 0;
                                if( yPos + height > desktop.getHeight() ) yPos = 0;
                            }
                            catch( java.beans.PropertyVetoException ex )
                            {
                                logger.error( ex.getMessage(), ex.fillInStackTrace() );
                            }
                        }
                    }
                }
            } );            
            // Cascading Internal Frames
            DnMenuItem mnWindowCascading    = new DnMenuItem( "window.cascading" );
            mnWindowCascading.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e) 
                {
                    JInternalFrame [] frames = desktop.getAllFrames();
                    int frameDistance = 25;
                    int xPos = 0;
                    int yPos = 0;
                    int width  = 300;
                    int height = 200;
                    for( int i = 0; i < frames.length; i++ )
                    {
                        if( ! frames[ i ].isIcon() )
                        {
                            try
                            {
                                frames[ i ].setMaximum( false );
                                frames[ i ].reshape( xPos, yPos,  width, height );
                                xPos += frameDistance;
                                yPos += frameDistance;
                                
                                if( xPos + width  > desktop.getWidth()  ) xPos = 0;
                                if( yPos + height > desktop.getHeight() ) yPos = 0;
                            }
                            catch( java.beans.PropertyVetoException ex )
                            {
                                logger.error( ex.getMessage(), ex.fillInStackTrace() );
                            }
                        }
                    }
                }
            } );                                    
            DnMenuItem mnWindowSymbols      = new DnMenuItem( "window.symbols" );
            //  Order Internal Frames symbols
            mnWindowSymbols.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e) 
                {
                    JInternalFrame [] frames = desktop.getAllFrames();
                    for( int i = 0; i < frames.length; i++ )
                    {
                        if( ! frames[ i ].isIcon() )
                        {
                            try
                            {
                                frames[ i ].setIcon( true );
                            }
                            // catch( PropertyVetoException ex )
                            catch( java.beans.PropertyVetoException ex )
                            {
                                logger.error( ex.getMessage(), ex.fillInStackTrace() );
                            }
                        }
                    }
                }
            } );            
            
            mnMenuWindow.add( mnWindowHorizontal  );
            mnMenuWindow.add( mnWindowVertical    );
            mnMenuWindow.add( mnWindowCascading   );
            mnMenuWindow.add( mnWindowSymbols );
            menu.add( mnMenuWindow );
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        //----------------------------------------------------------------------
        // Help menu
        //----------------------------------------------------------------------
        if( logger.isInfoEnabled() ) logger.info( "Build help menu" );
        try
        {
            mnMenuHelp = new JMenu( "Help" );
            mnMenuHelp.setMnemonic( KeyEvent.VK_H );
            
            DnMenuItem mnHelpHelp     = new DnMenuItem( "help.help" );
            mnHelpHelp.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e) 
                {
                    dnHelp();
                }
            } );            
            DnMenuItem mnHelpWeb      = new DnMenuItem( "help.web" );
            DnMenuItem mnHelpFaq      = new DnMenuItem( "help.faq" );
            mnHelpFaq.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e) 
                {
                    dnFAQ();
                }
            } );            
            DnMenuItem mnHelpAbout    = new DnMenuItem( "help.about" );
            mnHelpAbout.addActionListener( new ActionListener()
            {
                public void actionPerformed(ActionEvent e) 
                {
                    About();                    
                }
            } );            
            
            mnMenuHelp.add( mnHelpHelp  );
            mnMenuHelp.add( mnHelpFaq   );
            mnMenuHelp.add( mnHelpWeb   );
            mnMenuHelp.add( mnHelpAbout );
            menu.add( mnMenuHelp );
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
	setJMenuBar( menu );
        //--------------------------------------------------------------
	// Toolbar
        //--------------------------------------------------------------
	
        JToolBar B = new JToolBar();
	getContentPane().add(B, BorderLayout.NORTH);
        //----------------------------------------------------------------------
        // Toolbar export
        //----------------------------------------------------------------------
        if( logger.isInfoEnabled() ) logger.info( "Build Toolbar" );
        
	cmdExportTab = new JButton();
        cmdExportTab.setIcon( DiversityNavigatorProperties.getExportTabIcon() );
	cmdExportTab.setToolTipText( "Export table rows to UTF-8 file" );
	cmdExportTab.addActionListener( new ActionListener() 
        {
            public void actionPerformed(ActionEvent e) 
            {
                exportTab();
            }
        } );
	B.add( cmdExportTab );
        //----------------------------------------------------------------------
        // Toolbar export XML
        //----------------------------------------------------------------------
	cmdExportXml = new JButton();
        cmdExportXml.setIcon( DiversityNavigatorProperties.getExportXmlIcon() );
	cmdExportXml.setToolTipText( "Export table rows to XML file" );
	cmdExportXml.addActionListener( new ActionListener() 
        {
            public void actionPerformed(ActionEvent e) 
            {
                exportXml();
            }
        } );
	B.add( cmdExportXml );
        //----------------------------------------------------------------------
        // Toolbar import
        //----------------------------------------------------------------------
	cmdImportTab = new JButton();
        cmdImportTab.setIcon( DiversityNavigatorProperties.getImportTabIcon() );
	cmdImportTab.setToolTipText( "Import table rows from UTF-8 file. Do not save the rows" );
	cmdImportTab.addActionListener( new ActionListener() 
        {
            public void actionPerformed(ActionEvent e) 
            { 
                importTab(); 
            }
	});
	B.add( cmdImportTab );
        //----------------------------------------------------------------------
        // Toolbar import XML
        //----------------------------------------------------------------------
        
	cmdImportXml = new JButton();
        cmdImportXml.setIcon( DiversityNavigatorProperties.getImportXmlIcon() );
	cmdImportXml.setToolTipText( "Import table rows from XML file. Do not save the rows" );
	cmdImportXml.addActionListener( new ActionListener() 
        {
            public void actionPerformed(ActionEvent e) 
            { 
                importXml(); 
            }
	});
	B.add( cmdImportXml );
        //----------------------------------------------------------------------
        //----------------------------------------------------------------------
	cmdSaveTable = new JButton( DiversityNavigatorProperties.getSaveIcon() );
	cmdSaveTable.setToolTipText("Apply changes to database");
	cmdSaveTable.addActionListener( new ActionListener() 
        {
            public void actionPerformed(ActionEvent e) 
            {
                if (currentTable() != null) currentTable().save();
            }
	} );
	B.add( cmdSaveTable );
	B.add( Box.createHorizontalStrut(10) );
	cmdOpenSQL = new JButton( DiversityNavigatorProperties.getOpenIcon() );
	cmdOpenSQL.setToolTipText("Open SQL text file in Query editor");
	cmdOpenSQL.addActionListener( new ActionListener() 
        {
            public void actionPerformed(ActionEvent e) 
            {
		if (currentQuery() != null) currentQuery().openSQL();
            }
	} );
	B.add( cmdOpenSQL );
	cmdSaveSQL = new JButton( DiversityNavigatorProperties.getSaveIcon() );
	cmdSaveSQL.setToolTipText("Save Query in editor to a text file");
	cmdSaveSQL.addActionListener( new ActionListener() 
        {
            public void actionPerformed(ActionEvent e) 
            {
		if (currentQuery() != null) currentQuery().saveSQL();
            }
	} );
	B.add( cmdSaveSQL );
	cmdExecuteSQL = new JButton( DiversityNavigatorProperties.getExecuteIcon() );
	cmdExecuteSQL.setToolTipText("Execute Query and display results");
	cmdExecuteSQL.addActionListener( new ActionListener() 
        {
            public void actionPerformed(ActionEvent e) 
            {
		if (currentQuery() != null) currentQuery().executeQuery();
            }
	} );
	B.add( cmdExecuteSQL );
        
        //--------------------------------------------------------------
        // status bar
        //--------------------------------------------------------------
        if( logger.isInfoEnabled() ) logger.info( "Build status bar" );
        
	StatusBar = new Box(BoxLayout.X_AXIS);
	statConnection = new JLabel();
        // ClearLook
	// statConnection.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
	statQuery = new JLabel();
        // ClearLook
	//statQuery.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
	StatusBar.add( statConnection );
	StatusBar.add( Box.createHorizontalStrut(10) );
	StatusBar.add( statQuery );
        getContentPane().add(StatusBar,BorderLayout.SOUTH);
	setConnectionStatus( "not connected" );
	setQueryStatus( "no query" );
	recalcEnabled();
        setSize( new Dimension( 800,600 ) );
        // pack();        
	setVisible( true );	
         
        if( logger.isInfoEnabled() ) logger.info( "AboutDiversityNavigator" );        
        new AboutDiversityNavigator( this );
    }

    //----------------------------------------------------------------------
    //
    //----------------------------------------------------------------------
    public TableFrame currentTable() 
    {
        if( logger.isInfoEnabled() ) logger.info( "curentTable()" );
	try 
        {
            JInternalFrame frame = desktop.getSelectedFrame();
            if( frame == null ) 
                return null;
            if( Class.forName( "net.metadiversity.diversity.navigator.table.TableFrame" ).isInstance( frame )) 
                return (TableFrame) frame;
	} 
        catch(Exception ex ) 
        {
            ex.printStackTrace();
	}
	return null;
    }    
   //----------------------------------------------------------------------
   // ??? 
   //----------------------------------------------------------------------
    public void actionPerformed( ActionEvent e ) 
    {
        JMenuItem source = (JMenuItem)( e.getSource() );
        if( logger.isDebugEnabled() )
        {
            String s = "Action event detected. \n"
                   + "    Event source: " + source.getText()
                   + " (an instance of " +  getClassName(source) + ")";
        
            logger.debug( s );
        }        
        if( source.getText().equals( "Exit" ) )
        {
            if( logger.isInfoEnabled() ) logger.info( "Exiting DiversityNavigator" );
            System.exit( 0 );
        }                
    } 
    // Returns just the class name -- no package info.
    protected String getClassName(Object o) 
    {
        String classString = o.getClass().getName();
        int dotIndex = classString.lastIndexOf(".");
        return classString.substring( dotIndex+1 );
    } 
   //----------------------------------------------------------------------
   //----------------------------------------------------------------------
    public QueryFrame currentQuery() 
    {
	try 
        {
            JInternalFrame frame = desktop.getSelectedFrame();
            if( frame == null)
            {
                return null;
            }
            if( Class.forName( "net.metadiversity.diversity.navigator.QueryFrame").isInstance( frame )) 
            {
                return (QueryFrame) frame;
            }
	} 
        catch( Exception ex ) 
        {
            logger.error(  "currentQuery()" );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
	return null;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private void exportTab() 
    {
	try 
        {
            TableFrame tableFrame = currentTable();
            if( tableFrame != null ) 
            {
		JFileChooser fc = new JFileChooser();
		fc.setDialogType( JFileChooser.SAVE_DIALOG );
		int ret = fc.showSaveDialog( this );
		if( ret == JFileChooser.APPROVE_OPTION ) 
                {
                    java.io.File file = fc.getSelectedFile();
                    tableFrame.exportTabTable( file );
		}				
            }
	} 
        catch( Exception ex) 
        {
            logger.error(  "exportTab()" );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	}
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private void exportXml() 
    {
	try 
        {
            TableFrame tableFrame = currentTable();
            if( tableFrame != null ) 
            {
		JFileChooser fc = new JFileChooser();
		fc.setDialogType( JFileChooser.SAVE_DIALOG );
		int ret = fc.showSaveDialog( this );
		if( ret == JFileChooser.APPROVE_OPTION ) 
                {
                    java.io.File file = fc.getSelectedFile();
                    tableFrame.exportXmlTable( file );
		}				
            }
	} 
        catch( Exception ex) 
        {
            logger.error(  "exportXML()"  +ex.getLocalizedMessage() );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	}
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private void importTab() 
    {
	try 
        {
            TableFrame tableFrame = currentTable();
            if( tableFrame != null ) 
            {
		JFileChooser fc = new JFileChooser();
		fc.setDialogType( JFileChooser.OPEN_DIALOG );
		int ret = fc.showOpenDialog( this );
		if( ret == JFileChooser.APPROVE_OPTION ) 
                {
                    java.io.File file = fc.getSelectedFile();
                    tableFrame.importTabTable( file );
		}				
            }
	} 
        catch( Exception ex ) 
        {
            logger.error(  "importTab()"  +ex.getLocalizedMessage() );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	}
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private void importXml() 
    {
	try 
        {
            TableFrame tableFrame = currentTable();
            if( tableFrame != null ) 
            {
		JFileChooser fc = new JFileChooser();
		fc.setDialogType( JFileChooser.OPEN_DIALOG );
		int ret = fc.showOpenDialog( this );
		if( ret == JFileChooser.APPROVE_OPTION ) 
                {
                    java.io.File file = fc.getSelectedFile();
                    tableFrame.importXmlTable( file );
		}				
            }
	} 
        catch( Exception ex ) 
        {
            logger.error(  "importXML()"  + ex.getLocalizedMessage() );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	}
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void setConnectionStatus( String str ) 
    {
        // Conection status must be displayed, so we use debug FATAL level      
        logger.info( "ConnectionStatus : "  + str );
        
	statConnection.setText(" " + str  + " " );
	statConnection.setForeground( Color.black );
        messageOutputArea.append( str  + "\n" );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void setConnectionError( String str ) 
    {
        // ConectionErrorstatus must be displayed, so we use debug FATAL level        
        logger.error(  "ConnectionError : "  + str );
        
	statConnection.setText(" " + str  + " " );
	statConnection.setForeground( Color.red ) ;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void setQueryStatus( String str ) 
    {
        logger.info( "Query status: " + str );
      	statQuery.setText(" " + str  + " ");
	statQuery.setForeground( Color.black );
        messageOutputArea.append( str + "\n" );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void setQueryError( String str ) 
    {
        logger.info( "Query error: " + str );
	statQuery.setText(" " + str + " ");
	statQuery.setForeground( Color.red );                
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void setError(String str ) 
    {
        logger.error( "Error: " + str );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void recalcEnabled() 
    {
        if( logger.isDebugEnabled() )
        {
            logger.debug( "recalcEnabled()" );
        }
	cmdExportTab.setEnabled(    currentTable() != null);
	cmdImportTab.setEnabled(    currentTable() != null);
	cmdExportXml.setEnabled(    currentTable() != null);
	cmdImportXml.setEnabled(    currentTable() != null);
	cmdSaveTable.setEnabled(    currentTable() != null);
	mnTable.setEnabled(         currentTable() != null);

	mnQuery.setEnabled(         currentQuery() != null);
        cmdOpenSQL.setEnabled(      currentQuery() != null);
	cmdSaveSQL.setEnabled(      currentQuery() != null);
	cmdExecuteSQL.setEnabled(   currentQuery() != null);
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void openQuery( ConnectionInfo ci, java.sql.Connection con ) 
    {
        if( logger.isDebugEnabled() )
        {
            logger.debug( "openQuery()" );
        }
	QueryFrame queryFrame = new QueryFrame( ci, con, this);
	queryFrame.addInternalFrameListener( new javax.swing.event.InternalFrameAdapter() 
        {
            public void internalFrameActivated(javax.swing.event.InternalFrameEvent e) 
            {
                recalcEnabled();
            }
	});
	desktop.add( queryFrame );
	queryFrame.show();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void openTable( ConnectionInfo ci, java.sql.Connection con, String table ) 
    {
        if( logger.isDebugEnabled() )
        {
            logger.debug( "openTable" );
        }
	TableFrame tableFrame = new TableFrame( ci, con, table, this);
	tableFrame.addInternalFrameListener( new javax.swing.event.InternalFrameAdapter() 
        {
            public void internalFrameActivated( javax.swing.event.InternalFrameEvent ex ) 
            { 
                recalcEnabled();                         
            }
	});
	desktop.add( tableFrame );
	tableFrame.show();
    }
    //----------------------------------------------------------------------
    // Help
    //----------------------------------------------------------------------
    public void dnHelp() 
    {
        try 
        {
            HtmlViewer browser = new HtmlViewer( "Diversity Navigator Help", "index.html" );
            //  center gui in main frame
            java.awt.Point pos = getLocationOnScreen();
            pos.translate( (int) getWidth() / 2 - browser.getWidth() / 2, (int) getHeight() / 2 - browser.getHeight() / 2 );
            browser.setLocation( pos );
            browser.setVisible( true );   
            logger.debug( "start HelpViewer" );
            browser.start();
            // browser.run();
        }
        catch(Exception ex ) 
        {
            logger.error( "Can't open frame Help : " + ex.toString() );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void dnFAQ() 
    {
        try 
        {
            HtmlViewer browser = new HtmlViewer( "Diversity Navigator FAQ", "faq.html" );
            //  center gui in main frame
            java.awt.Point pos = getLocationOnScreen();
            pos.translate( (int) getWidth() / 2 - browser.getWidth() / 2, (int) getHeight() / 2 - browser.getHeight() / 2 );
            browser.setLocation( pos );
            logger.debug( "start HelpViewer" );
            browser.run();
        }
        catch(Exception ex ) 
        {
            logger.error( "Can't open frame FAQ : " + ex.toString() );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void About()
    {
        new AboutDiversityNavigator( this );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void openProject( String projectName ) 
    {
        logger.info( "Open project: " + projectName );
        JInternalFrame pluginFrame;
        Class c;                
        net.metadiversity.diversity.navigator.project.ProjectInfo pi;        
        pi = net.metadiversity.diversity.navigator.project.ProjectManager.getProjectInfoByName( projectName );
        if( pi != null )
        {    
            logger.info( "Load project class: " + pi.getProjectClassName() );
        }
        
        try 
        {
            c = Class.forName( pi.getProjectClassName() );
            pluginFrame = (JInternalFrame) c.newInstance();
            if( pluginFrame == null ) 
            { 
                logger.error( "Can't get new instance : " + pi.getProjectClassName() );
                return;
            }    
            pluginFrame.setTitle( projectName );
            pluginFrame.setClosable(true);
            pluginFrame.setIconifiable(true);
            pluginFrame.setMaximizable(true);
            pluginFrame.setResizable(true);
            pluginFrame.setVisible(true);
            pluginFrame.setSize( 640, 480 );
            desktop.add( pluginFrame );    
            pluginFrame.toFront();
        }
        catch(Exception ex ) 
        {
            logger.error( "Error while open project: " + projectName );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            DnOptionPane.showMessageDialog( null, "Error while open project: " + projectName, "Error" , DnOptionPane.ERROR_MESSAGE );
        }
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    void setFontType( int fontType )
    {
        try
        {
            com.jgoodies.looks.plastic.Plastic3DLookAndFeel plaf = new com.jgoodies.looks.plastic.Plastic3DLookAndFeel();
            switch( fontType )
            {
                case SYSTEM_FONT_HINT:    
                    plaf.setFontSizeHints( com.jgoodies.looks.FontSizeHints.SYSTEM );
                    break;
                case LARGE_FONT_HINT:     
                    plaf.setFontSizeHints( com.jgoodies.looks.FontSizeHints.LARGE );
                    break;
                case MIXED_FONT_HINT:     
                    plaf.setFontSizeHints( com.jgoodies.looks.FontSizeHints.MIXED );
                    break;
                case FIXED_FONT_HINT:     
                    plaf.setFontSizeHints( com.jgoodies.looks.FontSizeHints.FIXED );
                    break;
                case SMALL_FONT_HINT:     
                    plaf.setFontSizeHints( com.jgoodies.looks.FontSizeHints.SMALL );
                    break;
            }
            // User defined Font-Size
            // plaf.setFontSizeHints( new com.jgoodies.plaf.FontSizeHints( 14, 14, 16, 16 ) );
            // com.jgoodies.plaf.Options.setUseSystemFonts( true );
            // ((com.jgoodies.plaf.plastic.PlasticLookAndFeel) UIManager.getLookAndFeel()).setFontSizeHints( com.jgoodies.plaf.FontSizeHints.LARGE );
            plaf.setMyCurrentTheme( new com.jgoodies.looks.plastic.theme.SkyBlue() );
            UIManager.setLookAndFeel( plaf );       
            
            javax.swing.SwingUtilities.updateComponentTreeUI( this );
                        
        }
        catch( Exception ex )
        {
            logger.info( "Can't setup LookAndFeel" );
            // logger.info( "Class: com.jgoodies.plaf.plastic.Plastic3DLookAndFeel not found." );
        }
    }
}