/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package net.metadiversity.diversity.navigator;

import java.awt.*;
import java.util.*;

import java.sql.*;

import javax.swing.*;

import org.apache.log4j.Logger;

import net.metadiversity.diversity.navigator.table.*;
import net.metadiversity.diversity.navigator.connection.*;

public class QueryFrame extends JInternalFrame 
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3258411733583213881L;

    static Logger logger = Logger.getLogger( QueryFrame.class.getName() ); 
    
    // ??? private ConnectionInfo      ci;
    private Connection          con;
    private JTabbedPane         frames;
    private JPanel              queryPane;
    private JTextArea           edQuery;
    private JTextArea           output;
    private DiversityNavigator  mainWindow;
    int nData = 0;

    public QueryFrame( ConnectionInfo ci, Connection con, DiversityNavigator mainWindow ) 
    {
	super( ci.getName(), true, true, true, true);
	// ??? this.ci     = ci;
	this.con    = con;
	this.mainWindow = mainWindow;
	getContentPane().setLayout(new BorderLayout());
                
        //---------------------
        // Set Diversity Icon
        //----------------------
        this.setFrameIcon( DiversityNavigatorProperties.getDiversityIcon() );

        //--------------------------------------------------------------
        // frames
        //--------------------------------------------------------------
        frames = new JTabbedPane();
	getContentPane().add( frames, BorderLayout.CENTER );

        //--------------------------------------------------------------
        // Query Pane
        //--------------------------------------------------------------
	queryPane       = new JPanel();
	edQuery         = new JTextArea();
	JScrollPane qsp = new JScrollPane( edQuery );
        
	queryPane.setLayout( new BorderLayout() );
	queryPane.add( qsp, BorderLayout.CENTER );
	frames.add( queryPane, "Query" );

        //--------------------------------------------------------------
	// output
        //--------------------------------------------------------------
	JPanel POutput  = new JPanel();
	output          = new JTextArea();
	JScrollPane osp = new JScrollPane( output );
        
	output.setEditable( false );
	frames.add( osp, "output" );

        //--------------------------------------------------------------
	// Buttons bar
        //--------------------------------------------------------------
	JToolBar Buttons    = new JToolBar( "Query options" );
	JButton cmdOpenFile = new JButton( DiversityNavigatorProperties.getOpenIcon() );
	cmdOpenFile.setToolTipText( "Open SQL text file in Query editor" );
	cmdOpenFile.addActionListener( new java.awt.event.ActionListener() 
        {
            public void actionPerformed( java.awt.event.ActionEvent ev ) 
            {
		openSQL();
            }
	});
	Buttons.add( cmdOpenFile );
	JButton cmdSaveFile = new JButton( DiversityNavigatorProperties.getSaveIcon() );
	cmdSaveFile.setToolTipText( "Save Query in editor to a text file" );
	cmdSaveFile.addActionListener( new java.awt.event.ActionListener() 
        {
            public void actionPerformed( java.awt.event.ActionEvent ev ) 
            {
		saveSQL();
            }
	});
	Buttons.add( cmdSaveFile );
	Buttons.add( Box.createHorizontalStrut(10) );
	JButton cmdExecute = new JButton( DiversityNavigatorProperties.getExecuteIcon() );
	cmdExecute.setToolTipText( "Execute Query and display results" );
	cmdExecute.addActionListener( new java.awt.event.ActionListener() 
        {
            public void actionPerformed( java.awt.event.ActionEvent ev ) 
            {
		executeQuery();
            }
	});
	Buttons.add( cmdExecute );

	getContentPane().add(Buttons, BorderLayout.NORTH);	
	setBounds(0,0,400,300);
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public DiversityNavigator getMainWindow() 
    {
        return mainWindow;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void executeQuery() 
    {
	output.append( "\n----------- Sending Queries ------------\n" );
	frames.setSelectedIndex( 1 );
	for( Enumeration Enum = getQueries().elements(); Enum.hasMoreElements(); ) 
        {
            String SQL = (String) Enum.nextElement();
            
            if( SQL.toUpperCase().indexOf( "SELECT" ) >= 0 ) 
            {
                nData ++;
                output.append( SQL + "\n" );
                output.append( "sending output to Data" + nData + "\n\n" );

                JPanel resultsPane      = new JPanel();
                ResultsTableModel rtm   = new ResultsTableModel();
                
                resultsPane.setLayout( new BorderLayout() );
                
                // Sorter for table
                TableSorter sorter = new TableSorter( rtm ); 
                JTable results     = new JTable( sorter );
                results.setAutoResizeMode( JTable.AUTO_RESIZE_OFF ); 
                sorter.addMouseListenerToHeaderInTable( results );
                JScrollPane rsp = new JScrollPane( results );                
                resultsPane.add( rsp, BorderLayout.CENTER);
                frames.add( resultsPane, "Data" + nData);
                rtm.execute( con, SQL, getMainWindow() );
            } 
            else 
            {
                try 
                {
                    output.append( SQL + "\n" );		
                    Statement query = con.createStatement();
                    int result = query.executeUpdate( SQL );
                    output.append( result + " rows affected\n\n" );
                    query.close();
                } 
                catch( SQLException sql_ex  ) 
                {
                    logger.error( sql_ex.getMessage() );
                    getMainWindow().setQueryError( sql_ex.getMessage() );
                    output.append( "Error: " + sql_ex.getMessage() + "\n\n" );
                }
                catch( Exception ex ) 
                {
                    logger.error( ex.getMessage(), ex.fillInStackTrace() );
                }
            }
        }
    }    
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private Vector getQueries() 
    {
	Vector ret = new Vector();
	int i = 0;
	String st = edQuery.getText();
	int INSTRING = 1;
	int INQUERY = 2;
	int Status = INQUERY;
	char c;
	StringBuffer query = new StringBuffer();
	while( i < st.length() ) 
        {
            c = st.charAt(i);
            boolean add = true;
            if (c == '\'') 
            {
		if( Status == INSTRING ) 
                {
                    Status = INQUERY;
                }
		else 
                {
                    Status = INSTRING;
                }
            }
            if( (c == ';' && Status != INSTRING) ) 
            {
		add = false;
		String x = query.toString().trim();
		if( x.length() > 0 )
                {
                    ret.add( x );
                }
		query = new StringBuffer();
            }
            if( i == ( st.length() - 1 ) ) 
            {	
                add = false;
		query.append( c );
		String x = query.toString().trim();
		if( x.length() > 0 ) 
                {
                    ret.add( x );
                }
            }
            if( add )
            {
                query.append( c );
            }
            i++;
	}
	return ret;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void openSQL() 
    {
	JFileChooser fc = new JFileChooser();
	fc.setDialogType( JFileChooser.OPEN_DIALOG );
	int ret = fc.showOpenDialog(this);
		
        if( ret == JFileChooser.APPROVE_OPTION ) 
        {
            java.io.FileInputStream fis = null;
            try 
            {
                java.io.File file = fc.getSelectedFile();
		fis = new java.io.FileInputStream( file );
                byte[] BCode = new byte[ fis.available() ];
                fis.read(BCode);
                String str = new String( BCode );				
                fis.close();
		edQuery.setText( str );
            } 
            catch( Exception ex ) 
            {
                 logger.error( ex.getMessage(), ex.fillInStackTrace() );
                 if( fis != null )
                 {
                     try
                     {
                        fis.close();
                     }
                     catch( Exception eex ) 
                     {
                     }
                 }
            }
	}
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void saveSQL() 
    {
	JFileChooser fc = new JFileChooser();
	fc.setDialogType( JFileChooser.SAVE_DIALOG );
	int ret = fc.showSaveDialog(this);
	if( ret == JFileChooser.APPROVE_OPTION ) 
        {
            java.io.FileWriter fw = null;
            try 
            {
		fw = new java.io.FileWriter( fc.getSelectedFile() );
		String str =edQuery.getText();
		fw.write( str, 0, str.length() );
		fw.close();
            } 
            catch( Exception ex ) 
            {
                 logger.error( ex.getMessage(), ex.fillInStackTrace() );
                 if( fw != null )
                 {
                     try
                     {
                        fw.close();
                     }
                     catch( Exception eex ) 
                     {
                     }
                 }
            }
	}
    }
}

