/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * DBTreeModel.java
 *
 * Created on January 17, 2002, 4:32 PM
 */

package net.metadiversity.diversity.navigator.db;

import javax.swing.event.*;
import javax.swing.tree.*;
import java.util.*;

import org.apache.log4j.Logger;

import net.metadiversity.diversity.navigator.connection.*;


/**
 *
 * @author  didi
 */
public class DBTreeModel implements TreeModel 
{
    static Logger logger = Logger.getLogger( DBTreeModel.class.getName() ); 
    private DBNode root = null;
    private Vector Listeners = new Vector();
    private ConnectionsPane inPane;

    public DBTreeModel( ConnectionsPane inPane ) 
    {
        this.inPane = inPane;
        root = new DBNode( inPane, null, DBNode.NODE_TYPE_CAPTION_CONNECTIONS, "Connections" );
    }

    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void reload() 
    {
        root.reloadChilds();
        for( Enumeration Enum=Listeners.elements(); Enum.hasMoreElements(); ) 
        {
                TreeModelListener l = (TreeModelListener)Enum.nextElement();
                Object[] reloadFrom = {root};
                TreeModelEvent e = new TreeModelEvent(this, reloadFrom);
                l.treeStructureChanged(e);
        }
    }
    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void reloadSubTree(DBNode reloadFromNode) 
    {
        root.reloadChilds();
        for( Enumeration Enum=Listeners.elements(); Enum.hasMoreElements(); ) 
        {
                TreeModelListener l = (TreeModelListener)Enum.nextElement();
                Object[] reloadFrom = {reloadFromNode};
                TreeModelEvent e = new TreeModelEvent(this, reloadFrom);
                l.treeStructureChanged(e);
        }
    }
    //--------------------------------------------------------------------------
    // Adds a listener for the TreeModelEvent posted after the tree changes.
    //--------------------------------------------------------------------------
    public void addTreeModelListener( TreeModelListener listener ) 
    {
        Listeners.add( listener );
    }
    //--------------------------------------------------------------------------
    // Returns the child of parent at index index in the parent's child array.
    //--------------------------------------------------------------------------
    public Object getChild( Object parent, int index ) 
    {
        return (Object)( ( (DBNode) parent ).getChild( index ) );
    }
    //--------------------------------------------------------------------------
    // Returns the number of children of parent.
    //--------------------------------------------------------------------------
    public int getChildCount( Object parent ) 
    {
        return ( ( DBNode ) parent ).getChildCount();
    }
    //--------------------------------------------------------------------------
    // Returns the index of child in parent.
    //--------------------------------------------------------------------------
    public int getIndexOfChild( Object parent, Object child ) 
    {
        return ( ( DBNode ) parent ).getChildIndex( child );
    }
    //--------------------------------------------------------------------------
    // Returns true if the receiver is a leaf.
    //--------------------------------------------------------------------------
    public Object getRoot() 
    {
        return ( Object ) root; 
    }
    //--------------------------------------------------------------------------
    // Returns true if node is a leaf.
    //--------------------------------------------------------------------------
    public boolean isLeaf( Object node ) 
    {
        return ( ( DBNode ) node ).isLeaf();
    }
    //--------------------------------------------------------------------------
    // Removes a listener previously added with addTreeModelListener.
    //--------------------------------------------------------------------------
    public void removeTreeModelListener( TreeModelListener listener ) 
    {
        Listeners.remove( listener );
    }
    //--------------------------------------------------------------------------
    // Messaged when the user has altered the value for the item identified by path to newValue.
    //--------------------------------------------------------------------------
    public void valueForPathChanged( TreePath path, Object newValue ) 
    {
        return;
    }
}
