/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * CharacterStateSelctor.java
 *
 * Created on 13. Februar 2004, 11:42
 */

package net.metadiversity.diversity.navigator.gui.delta_editor;

import net.metadiversity.diversity.navigator.db.delta_editor.CharacterStateDescriptionEntry;
//
import org.apache.log4j.Logger;


/**
 *
 * @author  didi
 */
public class CharacterStateEditor extends javax.swing.JScrollPane implements java.awt.event.ActionListener
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3256719585087598899L;

    static Logger logger = Logger.getLogger( CharacterStateEditor.class.getName() ); 
    
    private javax.swing.JTable          characterStateEditor;
    private java.lang.String            characterStateText;
    //
    private CharacterStateEditorModel   characterStateEditorModel;
    
    private boolean debug = false;
  
    private boolean isEditable;

    // GUI SETTINGS
    
    static int              tableCellHorizontalAlignment;
    
    static boolean centerFlag = false; // true;
    
    /** Creates a new instance of CharacterStateSelctor */
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public CharacterStateEditor() 
    {
        characterStateEditor        = new javax.swing.JTable();
        characterStateText          = "Character State";
        characterStateEditorModel   = null;
        if( centerFlag )
        {
            tableCellHorizontalAlignment= javax.swing.SwingConstants.CENTER;
        }
        else
        {
            tableCellHorizontalAlignment= javax.swing.SwingConstants.LEFT;
        }
        initComponents();        
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public CharacterStateEditor(String characterStateName) 
    {
        characterStateText          = characterStateName;
        characterStateEditor        = new javax.swing.JTable();
        characterStateEditorModel   = null;
        if( centerFlag )
        {
            tableCellHorizontalAlignment= javax.swing.SwingConstants.CENTER;
        }
        else
        {
            tableCellHorizontalAlignment= javax.swing.SwingConstants.LEFT;
        }
        // ???
        // characterStateEditor.setPreferredScrollableViewportSize( new java.awt.Dimension( 400, 200 ) );
        
        initComponents();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    private void initComponents() 
    {          
        // set alignement for EditFields
        javax.swing.table.DefaultTableCellRenderer renderer = (javax.swing.table.DefaultTableCellRenderer) characterStateEditor.getDefaultRenderer( java.lang.String.class );
        // We use US-Local for the renderer, because double-values shall be like 1.0
        // renderer.setLocale( java.util.Locale.US );
        renderer.setHorizontalAlignment( tableCellHorizontalAlignment );
        // .setHorizontalAlignment(  );
        
        // Create a column model for character selection
        javax.swing.table.TableColumnModel selectionColumnModel = new javax.swing.table.DefaultTableColumnModel()
        {
            int col = 0;
            
            public void addColumn( javax.swing.table.TableColumn tc )
            {
                switch( col )
                {
                case 0: // CS
                    tc.setMinWidth( 50 );
                    tc.setMaxWidth( 50 );
                    break;
                case 1: // modifier   
                    tc.setMinWidth( 50 );
                    tc.setMaxWidth( 50 );
                    break;
                case 2: // X
                    tc.setMinWidth( 40 );
                    tc.setMaxWidth( 40 );
                    break;
                // Notes and Text as Comment
                case 3: // Text   
                case 4: // Notes  
                case 5: // Comment
                    tc.setMinWidth( 200 );
                    tc.setPreferredWidth( 200 );
                    tc.setMaxWidth( 1000 );
                    break;                        
                default:
                    tc.setMinWidth( 100 );
                    tc.setPreferredWidth( 100 );
                    tc.setMaxWidth( 1000 );
                    break;                        
                }
                super.addColumn( tc );
                col++;
            }
        };
        
        characterStateEditorModel = new CharacterStateEditorModel( characterStateText );
        characterStateEditor.setModel( characterStateEditorModel );
        characterStateEditor.setColumnModel( selectionColumnModel );
        characterStateEditor.createDefaultColumnsFromModel();
        
        // RESIZE
        characterStateEditor.setAutoResizeMode( javax.swing.JTable.AUTO_RESIZE_OFF );        
        setViewportView( characterStateEditor );
    }
    public CharacterStateEditorModel getModel()
    {
        return characterStateEditorModel;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setCharacterStatesDescription( java.util.Vector v )
    {
        java.util.Collections.sort( v );

        java.util.Iterator iter = v.listIterator();
        while( iter.hasNext() )
        {
            characterStateEditorModel.addCharacterStateDescriptionEntry( ( CharacterStateDescriptionEntry ) iter.next() );
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setEditable( boolean b )
    {
         isEditable = b;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public boolean isEditable()
    {
         return isEditable;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void actionPerformed(java.awt.event.ActionEvent actionEvent)
    {
        logger.debug( actionEvent );
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------    
    public void addDeltaEditorChangeListener( DeltaEditorChangeListener listener )
    {
        characterStateEditorModel.addDeltaEditorChangeListener( listener );
    }    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------    
    public void setVisible( boolean b )
    {
        super.setVisible( b );
        characterStateEditor.updateUI();
        characterStateEditor.setVisible( b );
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------    
    /**
     * @param args the command line arguments
     */
    public static void main( String args[] ) 
    {
        javax.swing.JFrame frame = new javax.swing.JFrame();
        CharacterStateSelector sel = new CharacterStateSelector();
                
        sel.setVisible( true );
        frame.setSize( 800, 600 );
        frame.getContentPane().setLayout( new java.awt.BorderLayout(0, 0));
        frame.setDefaultCloseOperation( javax.swing.JFrame.EXIT_ON_CLOSE );
        frame.getContentPane().add( sel,java.awt.BorderLayout.CENTER );
        
        frame.setVisible( true );
        frame.pack();
    }            
}
