/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package net.metadiversity.diversity.navigator.hibernate.bsm;

import java.io.Serializable;
import java.util.Set;
import org.apache.commons.lang.builder.ToStringBuilder;


/** 
 *        @hibernate.class
 *         table="TaxonNameExternalDatabase"
 *     
*/
public class TaxonNameExternalDatabase implements Serializable {

    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3257562923408044849L;

    /** identifier field */
    private Integer externalDatabaseId;

    /** nullable persistent field */
    private String externalDatabaseName;

    /** nullable persistent field */
    private String externalDatabaseVersion;

    /** nullable persistent field */
    private String rights;

    /** nullable persistent field */
    private String externalDatabaseAuthors;

    /** nullable persistent field */
    private String externalDatabaseUri;

    /** nullable persistent field */
    private String externalDatabaseInstitution;

    /** nullable persistent field */
    private String internalNotes;

    /** nullable persistent field */
    private String externalAttributeNameId;

    /** nullable persistent field */
    private Byte preferredSequence;

    /** nullable persistent field */
    private Boolean disabled;

    /** persistent field */
    private Set taxonNameExternalIds;

    /** full constructor */
    public TaxonNameExternalDatabase(Integer externalDatabaseId, String externalDatabaseName, String externalDatabaseVersion, String rights, String externalDatabaseAuthors, String externalDatabaseUri, String externalDatabaseInstitution, String internalNotes, String externalAttributeNameId, Byte preferredSequence, Boolean disabled, Set taxonNameExternalIds) {
        this.externalDatabaseId = externalDatabaseId;
        this.externalDatabaseName = externalDatabaseName;
        this.externalDatabaseVersion = externalDatabaseVersion;
        this.rights = rights;
        this.externalDatabaseAuthors = externalDatabaseAuthors;
        this.externalDatabaseUri = externalDatabaseUri;
        this.externalDatabaseInstitution = externalDatabaseInstitution;
        this.internalNotes = internalNotes;
        this.externalAttributeNameId = externalAttributeNameId;
        this.preferredSequence = preferredSequence;
        this.disabled = disabled;
        this.taxonNameExternalIds = taxonNameExternalIds;
    }

    /** default constructor */
    public TaxonNameExternalDatabase() {
    }

    /** minimal constructor */
    public TaxonNameExternalDatabase(Integer externalDatabaseId, Set taxonNameExternalIds) {
        this.externalDatabaseId = externalDatabaseId;
        this.taxonNameExternalIds = taxonNameExternalIds;
    }

    /** 
     *            @hibernate.id
     *             generator-class="assigned"
     *             type="java.lang.Integer"
     *             column="ExternalDatabaseID"
     *         
     */
    public Integer getExternalDatabaseId() {
        return this.externalDatabaseId;
    }

    public void setExternalDatabaseId(Integer externalDatabaseId) {
        this.externalDatabaseId = externalDatabaseId;
    }

    /** 
     *            @hibernate.property
     *             column="ExternalDatabaseName"
     *             length="100"
     *         
     */
    public String getExternalDatabaseName() {
        return this.externalDatabaseName;
    }

    public void setExternalDatabaseName(String externalDatabaseName) {
        this.externalDatabaseName = externalDatabaseName;
    }

    /** 
     *            @hibernate.property
     *             column="ExternalDatabaseVersion"
     *             length="255"
     *         
     */
    public String getExternalDatabaseVersion() {
        return this.externalDatabaseVersion;
    }

    public void setExternalDatabaseVersion(String externalDatabaseVersion) {
        this.externalDatabaseVersion = externalDatabaseVersion;
    }

    /** 
     *            @hibernate.property
     *             column="Rights"
     *             length="500"
     *         
     */
    public String getRights() {
        return this.rights;
    }

    public void setRights(String rights) {
        this.rights = rights;
    }

    /** 
     *            @hibernate.property
     *             column="ExternalDatabaseAuthors"
     *             length="200"
     *         
     */
    public String getExternalDatabaseAuthors() {
        return this.externalDatabaseAuthors;
    }

    public void setExternalDatabaseAuthors(String externalDatabaseAuthors) {
        this.externalDatabaseAuthors = externalDatabaseAuthors;
    }

    /** 
     *            @hibernate.property
     *             column="ExternalDatabaseURI"
     *             length="300"
     *         
     */
    public String getExternalDatabaseUri() {
        return this.externalDatabaseUri;
    }

    public void setExternalDatabaseUri(String externalDatabaseUri) {
        this.externalDatabaseUri = externalDatabaseUri;
    }

    /** 
     *            @hibernate.property
     *             column="ExternalDatabaseInstitution"
     *             length="300"
     *         
     */
    public String getExternalDatabaseInstitution() {
        return this.externalDatabaseInstitution;
    }

    public void setExternalDatabaseInstitution(String externalDatabaseInstitution) {
        this.externalDatabaseInstitution = externalDatabaseInstitution;
    }

    /** 
     *            @hibernate.property
     *             column="InternalNotes"
     *             length="1500"
     *         
     */
    public String getInternalNotes() {
        return this.internalNotes;
    }

    public void setInternalNotes(String internalNotes) {
        this.internalNotes = internalNotes;
    }

    /** 
     *            @hibernate.property
     *             column="ExternalAttribute_NameID"
     *             length="255"
     *         
     */
    public String getExternalAttributeNameId() {
        return this.externalAttributeNameId;
    }

    public void setExternalAttributeNameId(String externalAttributeNameId) {
        this.externalAttributeNameId = externalAttributeNameId;
    }

    /** 
     *            @hibernate.property
     *             column="PreferredSequence"
     *             length="3"
     *         
     */
    public Byte getPreferredSequence() {
        return this.preferredSequence;
    }

    public void setPreferredSequence(Byte preferredSequence) {
        this.preferredSequence = preferredSequence;
    }

    /** 
     *            @hibernate.property
     *             column="Disabled"
     *             length="1"
     *         
     */
    public Boolean getDisabled() {
        return this.disabled;
    }

    public void setDisabled(Boolean disabled) {
        this.disabled = disabled;
    }

    /** 
     *            @hibernate.set
     *             lazy="true"
     *             inverse="true"
     *             cascade="none"
     *            @hibernate.collection-key
     *             column="ExternalDatabaseID"
     *            @hibernate.collection-one-to-many
     *             class="net.metadiversity.diversity.navigator.hibernate.bsm.TaxonNameExternalId"
     *         
     */
    public Set getTaxonNameExternalIds() {
        return this.taxonNameExternalIds;
    }

    public void setTaxonNameExternalIds(Set taxonNameExternalIds) {
        this.taxonNameExternalIds = taxonNameExternalIds;
    }

    public String toString() {
        return new ToStringBuilder(this)
            .append("externalDatabaseId", getExternalDatabaseId())
            .toString();
    }

}
