/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * WriteUnicodeFile.java
 *
 * Created on March 7, 2002, 1:50 PM
 */

package net.metadiversity.diversity.navigator.io;

import org.apache.log4j.*;
/**
 *
 * @author  didi
 */
public class WriteUnicodeFile 
{
    static Logger logger = Logger.getLogger( WriteUnicodeFile.class.getName() );
    
java.io.FileOutputStream    fos;
java.io.OutputStreamWriter  osw; 
java.io.BufferedWriter      output;

java.lang.String            encoding;
java.lang.String            filename;

    public WriteUnicodeFile() 
    {
        setEncoding( "UTF8" );
        filename = null;        
    }
    //--------------------------------------------------------------------------    
    public WriteUnicodeFile( String filename ) 
    {
        setEncoding( "UTF8" );
        setFilename( filename );        
    }
    //--------------------------------------------------------------------------    
    public WriteUnicodeFile( String filename, String data ) 
    {
        setEncoding( "UTF8" );
        setFilename( filename );
        writeData( data );
    }
    //--------------------------------------------------------------------------    
    public WriteUnicodeFile( String filename, String data, String encoding )
    {
        setEncoding( encoding );
        setFilename( filename );
        writeData( data );        
    }
    //--------------------------------------------------------------------------    
    private boolean writeData( String data ) 
    {
        try
        {
            fos = new java.io.FileOutputStream( filename );
            osw = new java.io.OutputStreamWriter( fos, encoding );
            output  = new java.io.BufferedWriter( osw );
            
            // write data to file            
            try
            {
                output.write( data );
                output.close();
            }
            catch( java.io.IOException ex )
            {
                logger.error( ex.getMessage(), ex.fillInStackTrace() );
                // throw ex;
                return false;
            }
        }    
        catch( java.io.FileNotFoundException ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            // throw ex;
            return false;
        }
        catch( java.io.UnsupportedEncodingException ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            // throw ex;
            return false;
       }
       return true; 
    }
    //--------------------------------------------------------------------------
    public void setEncoding(String encoding) 
    {
        // we use BufferdReader for inport file reading
        //
        // posible encodings are:
        //
        // "US-ASCII"         "Seven-bit ASCII, a.k.a. ISO646-US, a.k.a. the Basic Latin block of the Unicode character set"
        // "ISO-8859-1"       "ISO Latin Alphabet No. 1, a.k.a. ISO-LATIN-1"
        // "UTF-8"            "Eight-bit Unicode Transformation Format"
        // "UTF-16BE"         "Sixteen-bit Unicode Transformation Format, big-endian byte order"
        // "UTF-16LE"         "Sixteen-bit Unicode Transformation Format, little-endian byte order"
        // "UTF-16"           "Sixteen-bit Unicode Transformation Format, byte order specified by a mandatory initial byte-order mark (either order accepted on input, big-endian used on output)"
        
        this.encoding = encoding;
    }
    //--------------------------------------------------------------------------
    public void setFilename( String filename )
    {
        this.filename = filename;
    }    
    //--------------------------------------------------------------------------
}
