/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * FormatUnit.java
 *
 * Created on July 19, 2006, 2:19 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.metadiversity.diversity.navigator.util;

/**
 *
 * @author didi
 */
public class FormatUnit 
{
    // http://physics.nist.gov/cuu/Units/prefixes.html
    
  /* Table 5.  SI prefixes
    Factor 	Name  	Symbol
    1024 	yotta 	Y
    1021 	zetta 	Z
    1018 	exa 	E
    1015 	peta 	P
    1012 	tera 	T
    109 	giga 	G
    106 	mega 	M
    103 	kilo 	k
    102 	hecto 	h
    101 	deka 	da
    10-1 	deci 	d
    10-2 	centi 	c
    10-3 	milli 	m
    10-6 	micro 	µ
    10-9 	nano 	n
    10-12 	pico 	p
    10-15 	femto 	f
    10-18 	atto 	a
    10-21 	zepto 	z
    10-24 	yocto 	y
     */
    final static String prefix[] = { "Y","Z","E","P","T","G","M","k","h","da","d","c","m","µ","n","p","f","a","z","y" };
    
    /* SI units
    mass                        kilogram       	kg
    time                        second          s
    electric current            ampere          A
    thermodynamic temperature   kelvin          K
    amount of substance 	mole            mol
l   uminous intensity           candela         cd
     */
    final static String[] units = { "m", "kg", "s", "A", "K", "mol", "cd", "min","h","g","L","T","bar" };
    
    /* SI derived unit
     * 
     Table 3.  SI derived units with special names and symbols
	SI derived unit
    Derived quantity 	Name 	Symbol   	Expression  

     plane angle 	radian (a)          rad 	  - 	m·m-1 = 1 (b)
    solid angle 	steradian (a)       sr (c) 	  - 	m2·m-2 = 1 (b)
    frequency           hertz               Hz 	  - 	s-1
    force               newton              N 	  - 	m·kg·s-2
    pressure, stress 	pascal              Pa 	N/m2 	m-1·kg·s-2
    energy, work, quantity of heat  joule   J 	N·m 	m2·kg·s-2
    power, radiant flux 	watt        W 	J/s 	m2·kg·s-3
    electric charge, quantity of electricity 	coulomb 	C 	  - 	s·A
    electric potential difference,
    electromotive force 	volt        V 	W/A 	m2·kg·s-3·A-1
    capacitance 	farad               F 	C/V 	m-2·kg-1·s4·A2
    electric resistance 	ohm         Omega 	V/A 	m2·kg·s-3·A-2
    electric conductance 	siemens     S 	A/V 	m-2·kg-1·s3·A2
    magnetic flux 	weber               Wb 	V·s 	m2·kg·s-2·A-1
    magnetic flux density 	tesla       T 	Wb/m2 	kg·s-2·A-1
    inductance 	henry                       H 	Wb/A 	m2·kg·s-2·A-2
    Celsius temperature	degree Celsius      °C 	  - 	K
    luminous flux 	lumen               lm 	cd·sr (c) 	m2·m-2·cd = cd
    illuminance 	lux                 lx 	lm/m2 	m2·m-4·cd = m-2·cd
    activity (of a radionuclide) becquerel  Bq 	  - 	s-1
    absorbed dose, specific energy (imparted), kerma 	gray 	Gy 	J/kg 	m2·s-2
    dose equivalent (d) 	sievert     Sv 	J/kg 	m2·s-2
    catalytic activity 	katal               kat 		s-1·mol
     */

    final static String derived[] = {"rad","sr","Hz","N","Pa","J","W","C","V","F","Omega","S","Wb","T","H","°C","lm","lx","Bq","Gy","Sv","kat",};
    
    /*
    178	²	&#178;	&sup2;  HEX: b3
    179	³	&#179;	&sup3;  HEX: b4
    */
    
    final static String[] exp = { "\u00b3", "\u00b4" };
    
    final static String[] delimiter = { "/", " ", "*", "°", "·" };

    private boolean debug = false;
    
    /**
     * Creates a new instance of FormatUnit
     */
    public FormatUnit() 
    {
    }
    
    public void show()
    {
        for( int i = 32; i < 2048; i++ )
        {
            char ch = (char) i;
            if( debug ) System.out.println( i + "\t" + Integer.toHexString( i ) + "\t --> " + ch );
        }
        
    }
    
    public String format( String str )
    {
        String f = "[";
        String lastPrefix = null;
        int pos = 0;
        int i;
        int p = 0;
        int u = 0;
        int e = 0;
        int der = 0;
        int del = 0;
        int errors = 0;
        
        if( debug ) System.out.println( str );
        while( pos < str.length() )
        {
            p = u = e = der = del = 0;
            
            for( i = 0; i < prefix.length; i++ )
            {
                p = java.lang.Math.max( str.substring( pos ).startsWith( prefix[ i ] ) ? prefix[ i ].length() : 0 , p );
            }
            for( i = 0; i < units.length; i++ )
            {
                u = java.lang.Math.max( str.substring( pos ).startsWith( units[ i ] ) ? units[ i ].length() : 0 , u );
            }
            for( i = 0; i < derived.length; i++ )
            {
                der = java.lang.Math.max( str.substring( pos ).startsWith( derived[ i ] ) ? derived[ i ].length() : 0 , der );
            }

            for( i = 0; i < exp.length; i++ )
            {
                e = java.lang.Math.max( str.substring( pos ).startsWith( exp[ i ] ) ? exp[ i ].length() : 0 , e );
            }
            for( i = 0; i < delimiter.length; i++ )
            {
                del = java.lang.Math.max( str.substring( pos ).startsWith( delimiter[ i ] ) ? delimiter[ i ].length() : 0 , del );
            }
            
            int m; 
            m = java.lang.Math.max( p, u );
            m = java.lang.Math.max( u, m );
            m = java.lang.Math.max( der, m );
            m = java.lang.Math.max( e, m );
            m = java.lang.Math.max( del, m );
            
            if( m == p )
            {
                // Problem [mm] Milli + Meter
                String pre = str.substring( pos, pos + m );
                if( lastPrefix == null )
                {    
                    lastPrefix = pre;
                }    
                else if( pre.compareTo( "m" ) == 0 )
                {
                    lastPrefix = null;
                }
                else
                {
                    m = -1;  // it is not posible to have 2 Prefixes --> Error
                }
            }
            else
            {
                lastPrefix = null;
            }
            
            if( m < 1 )
            {
                f += "#";
                errors++;
                pos++;
            }
            else
            {
                f += str.substring( pos, pos + m );
                pos += m;
            }
            
            if( debug ) System.out.println( f + "] " + str.substring( pos ) + " p= "+ p + " u= " + u + " der= " + der + " e= " + e + " del= " + del );
        } 
        return (errors > 0) ? "" : f + "]";
    }
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) 
    {
         FormatUnit fu = new FormatUnit();
         
         // Used UNITS in LIASlight
         
         System.out.println( fu.format( "-transversally septate" ) );
         System.out.println( fu.format( "mm wide" )   );
         System.out.println( fu.format( "per ascus" ) );
         System.out.println( fu.format( "µm long" )   );
         System.out.println( fu.format( "µm wide" )   );
         
         // Tests
         
         System.out.println( fu.format( "mm" )   );
         System.out.println( fu.format( "Mg A mol cd km * kg / mm * s" ) );
         System.out.println( fu.format( "µm" )   );
         System.out.println( fu.format( "µmdaddadammm * s" )   );
    }
    
}
