/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * CharacterStateSelctor.java
 *
 * Created on 13. Februar 2004, 11:42
 */

package net.metadiversity.diversity.navigator.gui.delta_editor;

import net.metadiversity.diversity.navigator.db.delta_editor.CharacterStateElement;
import net.metadiversity.diversity.navigator.db.delta_editor.CharacterStateDescriptionEntry;
//
import org.apache.log4j.Logger;


/**
 *
 * @author  didi
 */
// public class CharacterStateSelector extends javax.swing.JScrollPane
public class CharacterStateSelector extends javax.swing.JPanel
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3258133557007495987L;

    static Logger logger = Logger.getLogger( CharacterStateSelector.class.getName() ); 
    
    private javax.swing.JScrollPane characterStateScrollPane;
    private javax.swing.JTable      characterStateTable;
    private java.lang.String        characterStateText;
    //
    private CharacterStateModel     characterStateModel;
    private javax.swing.JLabel      typeUnitLabel;
    
    private final boolean debug = true;
    
    /** Creates a new instance of CharacterStateSelctor */
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public CharacterStateSelector()     
    {
        characterStateScrollPane    = new javax.swing.JScrollPane();
        characterStateTable         = new javax.swing.JTable();
        characterStateText          = "Character State";
        characterStateModel         = null;
        typeUnitLabel               = null;
        initComponents();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public CharacterStateSelector( String characterStateName, String typeUnitString ) 
    {
        characterStateScrollPane    = new javax.swing.JScrollPane();
        characterStateText          = characterStateName;
        characterStateTable         = new javax.swing.JTable();
        characterStateModel         = null;
        if( typeUnitString != null )
        {
            typeUnitLabel           = new javax.swing.JLabel( typeUnitString );
        }
        initComponents();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    private void initComponents() 
    {                   
        setLayout( new java.awt.BorderLayout() );
        // Create a column model for character selection
        javax.swing.table.TableColumnModel selectionColumnModel = new javax.swing.table.DefaultTableColumnModel()
        {
            int col = 0;
            
            public void addColumn( javax.swing.table.TableColumn tc )
            {
                switch( col++ )
                {
                case 1:
                    tc.setMinWidth( 40 );
                    tc.setMaxWidth( 40 );
                    super.addColumn( tc );
                    break;
                case 2:   
                    tc.setMinWidth( 100 );
                    tc.setPreferredWidth( 300 );
                    tc.setMaxWidth( 1000 );
                    super.addColumn( tc );
                    break;
                default :
                    tc.setMinWidth( 25 );
                    tc.setMaxWidth( 25 );
                    super.addColumn( tc );
                    break;
                }
            }
        };
        
        characterStateModel = new CharacterStateModel( characterStateText );
        // 
        characterStateTable.setModel( characterStateModel );
        characterStateTable.setColumnModel( selectionColumnModel );
        characterStateTable.createDefaultColumnsFromModel();
        
        // RESIZE
        characterStateTable.setAutoResizeMode( javax.swing.JTable.AUTO_RESIZE_OFF );        
        // characterStateScrollPane.setViewportView( characterStateTable );
        // add( characterStateScrollPane, java.awt.BorderLayout.CENTER );
        characterStateScrollPane.setViewportView( characterStateTable );
        //
        if( typeUnitLabel != null )
        {
            add( typeUnitLabel, java.awt.BorderLayout.NORTH );
        }    
        add( characterStateScrollPane, java.awt.BorderLayout.CENTER ); 
    }    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setCharacterStates( java.util.Vector v )
    {
        java.util.Iterator iter = v.listIterator();
        while( iter.hasNext() )
        {
            characterStateModel.addCharacterStateElement( ( CharacterStateElement ) iter.next() );
        }
    }
    //--------------------------------------------------------------------------
    // formated String x/y/z... (not used in this version)
    //--------------------------------------------------------------------------
    /*
    public void initCharacterSelection( java.lang.String str )
    {
        if( debug ) logger.debug( "initCharacterSelection( " + str + " )" );        
        java.util.StringTokenizer st = new java.util.StringTokenizer( str, "/", false );        
        while( st.hasMoreTokens() )
        {
            java.lang.String selection = st.nextToken();            
            // select this selection           
            characterStateModel.setCaracterStateSeletion( true, selection );
        }        
    }
     */
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setCharacterStatesDescription( java.util.Vector v )
    {
        java.util.Iterator iter = v.listIterator();
        while( iter.hasNext() )
        {
            String selection = (( CharacterStateDescriptionEntry ) iter.next() ).getCS();
            characterStateModel.setCaracterStateSeletion( true, selection );
        }
    }
    //--------------------------------------------------------------------------
    // Add listeners
    //--------------------------------------------------------------------------
    public void addDeltaEditorChangeListener( DeltaEditorChangeListener listener )
    {
        characterStateModel.addDeltaEditorChangeListener( listener );
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    /**
     * @param args the command line arguments
     */
    public static void main( String args[] ) 
    {
        javax.swing.JFrame frame = new javax.swing.JFrame();
        CharacterStateSelector sel = new CharacterStateSelector();
                
        sel.setVisible( true );
        frame.setSize( 800, 600 );
        frame.getContentPane().setLayout( new java.awt.BorderLayout(0, 0));
        frame.setDefaultCloseOperation( javax.swing.JFrame.EXIT_ON_CLOSE );
        frame.getContentPane().add( sel,java.awt.BorderLayout.CENTER );
        
        frame.setVisible( true );
        frame.pack();
    }        
}
