/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package net.metadiversity.diversity.navigator.hibernate.bsm;

import java.io.Serializable;
import java.util.Set;
import org.apache.commons.lang.builder.ToStringBuilder;


/** 
 *        @hibernate.class
 *         table="ProjectProxy"
 *     
*/
public class ProjectProxy implements Serializable {

    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3760558672446568241L;

    /** identifier field */
    private String project;

    /** persistent field */
    private int projectId;

    /** persistent field */
    private Set taxonHierarchies;

    /** persistent field */
    private Set taxonAcceptedNames;

    /** persistent field */
    private Set taxonSynonymies;

    /** full constructor */
    public ProjectProxy(String project, int projectId, Set taxonHierarchies, Set taxonAcceptedNames, Set taxonSynonymies) {
        this.project = project;
        this.projectId = projectId;
        this.taxonHierarchies = taxonHierarchies;
        this.taxonAcceptedNames = taxonAcceptedNames;
        this.taxonSynonymies = taxonSynonymies;
    }

    /** default constructor */
    public ProjectProxy() {
    }

    /** 
     *            @hibernate.id
     *             generator-class="assigned"
     *             type="java.lang.String"
     *             column="Project"
     *         
     */
    public String getProject() {
        return this.project;
    }

    public void setProject(String project) {
        this.project = project;
    }

    /** 
     *            @hibernate.property
     *             column="ProjectID"
     *             length="10"
     *             not-null="true"
     *         
     */
    public int getProjectId() {
        return this.projectId;
    }

    public void setProjectId(int projectId) {
        this.projectId = projectId;
    }

    /** 
     *            @hibernate.set
     *             lazy="true"
     *             inverse="true"
     *             cascade="none"
     *            @hibernate.collection-key
     *             column="Project"
     *            @hibernate.collection-one-to-many
     *             class="net.metadiversity.diversity.navigator.hibernate.bsm.TaxonHierarchy"
     *         
     */
    public Set getTaxonHierarchies() {
        return this.taxonHierarchies;
    }

    public void setTaxonHierarchies(Set taxonHierarchies) {
        this.taxonHierarchies = taxonHierarchies;
    }

    /** 
     *            @hibernate.set
     *             lazy="true"
     *             inverse="true"
     *             cascade="none"
     *            @hibernate.collection-key
     *             column="Project"
     *            @hibernate.collection-one-to-many
     *             class="net.metadiversity.diversity.navigator.hibernate.bsm.TaxonAcceptedName"
     *         
     */
    public Set getTaxonAcceptedNames() {
        return this.taxonAcceptedNames;
    }

    public void setTaxonAcceptedNames(Set taxonAcceptedNames) {
        this.taxonAcceptedNames = taxonAcceptedNames;
    }

    /** 
     *            @hibernate.set
     *             lazy="true"
     *             inverse="true"
     *             cascade="none"
     *            @hibernate.collection-key
     *             column="Project"
     *            @hibernate.collection-one-to-many
     *             class="net.metadiversity.diversity.navigator.hibernate.bsm.TaxonSynonymy"
     *         
     */
    public Set getTaxonSynonymies() {
        return this.taxonSynonymies;
    }

    public void setTaxonSynonymies(Set taxonSynonymies) {
        this.taxonSynonymies = taxonSynonymies;
    }

    public String toString() {
        return new ToStringBuilder(this)
            .append("project", getProject())
            .toString();
    }

}
