/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * DnDatabase.java
 *
 * Created on 28. Juli 2003, 11:50
 */

package net.metadiversity.diversity.navigator.project;

import org.apache.log4j.Logger;

import net.metadiversity.diversity.navigator.connection.ConnectionsManager;
import net.metadiversity.diversity.navigator.connection.ConnectionInfo;
/**
 *
 * @author  Dieter Neubacher
 */
public class DnDatabase
{
    static Logger logger = Logger.getLogger( DnDatabase.class.getName() );
    
    private java.lang.String    projectName;
    private java.lang.String    dbGroupName;
    private java.lang.String    databaseName;
    
    // ??? private java.lang.String    description; 
    
    private java.util.LinkedHashMap dnDbTables;
    private ConnectionInfo ci;
    
    private static final String POSTGRES_SYSTEM_TABLE_PREFIX_1 = "pga_";
    private static final String POSTGRES_SYSTEM_TABLE_PREFIX_2 = "sql_";
    
    /** Creates a new instance of DnDatabase */
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public DnDatabase( java.lang.String connectionName ) throws java.sql.SQLException
    {
        this.projectName    = null;
        this.dbGroupName    = null;
        databaseName = connectionName;
        dnDbTables   = new java.util.LinkedHashMap();
        // get conection info for this database       
        ci = ConnectionsManager.getConnectionInfoByName( databaseName );        
        // init tabels
        initTables();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public java.lang.String getName()
    {
        return databaseName;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    private void addDnDbTable( java.lang.String dbTableName, DnDbTable table ) throws java.sql.SQLException
    {
        dnDbTables.put( dbTableName, table );
        return;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public DnDbTable getDnTableByName( java.lang.String dnDbTableName )
    {
        return ( DnDbTable ) dnDbTables.get( dnDbTableName );
    }    
    /**
     * Get all TableNames from this database 
    */
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    private void initTables() throws java.sql.SQLException
    {
        logger.debug( "initTables()" );
        java.util.Vector tableNames = new java.util.Vector();
        java.sql.ResultSet mrs = null;
        try
        {
            java.sql.Connection conn = ci.getConnection();
            java.sql.DatabaseMetaData meta = conn.getMetaData();
            
            mrs = meta.getTables( null, null, null, new String[] { "TABLE" } );
            while( mrs.next() )
            {
                java.lang.String tablename = mrs.getString( 3 );
                // if it is not a postgresql system table, add table
                if( ! (tablename.toLowerCase().startsWith( POSTGRES_SYSTEM_TABLE_PREFIX_1 ) || tablename.toLowerCase().startsWith( POSTGRES_SYSTEM_TABLE_PREFIX_2 )) )
                {
                    tableNames.add( tablename );
                    logger.debug( tablename );
                }    
            }
            mrs.close();
            // add all tables 
            for( int i = 0; i < tableNames.size(); i++ )
            {   
                addDnDbTable( (java.lang.String)  tableNames.get( i ), new DnDbTable( meta, databaseName, (java.lang.String) tableNames.get( i ) ) ); 
            }    
        }
        catch( Exception ex )
        {
             // javax.swing.JOptionPane.showMessageDialog( this, ex );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        finally
        {
            if( mrs != null ) mrs.close();
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public int getChildCount()
    {
        return dnDbTables.size();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public java.lang.String getChild( int i )
    {
        java.util.Iterator iter = dnDbTables.keySet().iterator();
        int ii = 0;
        Object obj = null;
        while( iter.hasNext() )
        {
            obj = iter.next();
            if( i == ii )
            {
                break;
            }
            ii++;
        }
        return obj.toString();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
}
