/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * DnDbProjectRoot.java
 *
 * Created on 29. Juli 2003, 12:51
 */

package net.metadiversity.diversity.navigator.project;

import org.apache.log4j.Logger;

/**
 *
 * @author  Dieter Neubacher
 */
public class DnDbProjectRoot
{
    static Logger logger = Logger.getLogger( DnDbProjectRoot.class.getName() );
    
    public static final int LEVEL_ROOT      = 0;
    public static final int LEVEL_PROJECT   = 1;
    public static final int LEVEL_DB_GROUP  = 2;
    public static final int LEVEL_DATABASE  = 3;
    public static final int LEVEL_TABLE     = 4;
    public static final int LEVEL_FIELD     = 5;
    
    private java.util.LinkedHashMap dnDbProjects;
    
    /** Creates a new instance of DnDbProjectRoot */
    public DnDbProjectRoot()
    {
        // logger.setLevel( Level.DEBUG );
        dnDbProjects = new java.util.LinkedHashMap();
    }
    public void addDnProject( DnDbProject dbProject )
    {
        dnDbProjects.put( dbProject.getName(), dbProject );
        return;
    }    
    public DnDbProject getDnProjectByName( java.lang.String dnDbProject )
    {
        return ( DnDbProject ) dnDbProjects.get( dnDbProject );
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public int getChildCount()
    {
        return dnDbProjects.size();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public java.lang.String getChild( int i )
    {
        java.util.Iterator iter = dnDbProjects.keySet().iterator();
        int ii = 0;
        Object obj = null;
        while( iter.hasNext() )
        {
            obj = iter.next();
            if( i == ii )
            {
                break;
            }
            ii++;
        }
        return obj.toString();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public int getChildCount( java.lang.String node )
    {
        if( logger.isDebugEnabled() ) 
        {    
            logger.debug( "getChildCount( " + node + " )" );
        }
        
        java.util.Map childMap;
        
        DnDbProject project     = null;
        DnDbGroup   group       = null;
        DnDatabase  database    = null;
        DnDbTable   table       = null;
        DnDbField   field       = null;
        
        int level = 0;
        int pos = 0;
        int count = 0;
        
        java.lang.String str = node.toString();
        while( ( pos = str.indexOf( '.' ) ) > 0 )
        {
            level++;
            str = str.substring( ++pos );
            int nextPos = str.indexOf( '.' );
            java.lang.String name;
            if( nextPos > 0 )
            {
                name = str.substring( 0, nextPos );
            }
            else
            {
                name = str;
            }
            if( logger.isDebugEnabled() ) 
            {
                logger.debug( "str: " + str + " name: " + name );
            }    
            switch( level )
            {
                case LEVEL_ROOT :
                    childMap    = dnDbProjects;
                    break;
                case LEVEL_PROJECT :
                    project     = getDnProjectByName( name );
                    break;
                case LEVEL_DB_GROUP :
                    group       = project.getDnGroupByName( name );
                    break;
                case LEVEL_DATABASE :
                    database    = group.getDnDatabaseByName( name );
                    break;
                case LEVEL_TABLE :
                    table       = database.getDnTableByName( name );
                    break;
                case LEVEL_FIELD :
                    field       = table.getDnFieldByName( name );
                    break;
            }                
        }    
        switch( level )
        {
            case LEVEL_ROOT :
                if( dnDbProjects != null )  return getChildCount();
                break;
            case LEVEL_PROJECT :
                if( project != null )       return project.getChildCount();
                break;
            case LEVEL_DB_GROUP :
                if( group != null )         return group.getChildCount();
                break;
            case LEVEL_DATABASE :
                if( database != null )      return database.getChildCount();
                break;
            case LEVEL_TABLE :
                if( table != null )         return table.getChildCount();   
                break;
            case LEVEL_FIELD :
                return 0;
        }
        return 0;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public java.lang.String getChild( java.lang.String node, int index  )
    {
        if( logger.isDebugEnabled() ) logger.debug( "getChild( " + node + " index: " + index + " )" );
        
        java.util.Map childMap;
        
        DnDbProject project     = null;
        DnDbGroup   group       = null;
        DnDatabase  database    = null;
        DnDbTable   table       = null;
        DnDbField   field       = null;
        
        int level = 0;
        int pos = 0;
        int count = 0;
        
        java.lang.String str = node.toString();
        while( ( pos = str.indexOf( '.' ) ) > 0 )
        {
            level++;
            str = str.substring( ++pos );
            int nextPos = str.indexOf( '.' );
            java.lang.String name;
            if( nextPos > 0 )
            {
                name = str.substring( 0, nextPos );
            }
            else
            {
                name = str;
            }
            if( logger.isDebugEnabled() ) 
            {
                logger.debug( "str: " + str + " name: " + name );
            }    
            switch( level )
            {
                case LEVEL_ROOT :
                    childMap    = dnDbProjects;
                    break;
                case LEVEL_PROJECT :
                    project     = getDnProjectByName( name );
                    break;
                case LEVEL_DB_GROUP :
                    group       = project.getDnGroupByName( name );
                    break;
                case LEVEL_DATABASE :
                    database    = group.getDnDatabaseByName( name );
                    break;
                case LEVEL_TABLE :
                    table       = database.getDnTableByName( name );
                    break;
                case LEVEL_FIELD :
                    field       = table.getDnFieldByName( name );
                    break;
            }                
        }    
        switch( level )
        {
            case LEVEL_ROOT :
                if( dnDbProjects != null )  return node.toString() + "." + getChild( index );
                break;
            case LEVEL_PROJECT :
                if( project != null )       return node.toString() + "." + project.getChild( index );
                break;
            case LEVEL_DB_GROUP :
                if( group != null )         return node.toString() + "." + group.getChild( index );
                break;
            case LEVEL_DATABASE :
                if( database != null )      return node.toString() + "." + database.getChild( index );
                break;
            case LEVEL_TABLE :
                if( table != null )         return node.toString() + "." + table.getChild( index );   
                break;
            case LEVEL_FIELD :                
                return null;
        }
        return null;
    }
}
