/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package net.metadiversity.diversity.navigator.project;

import java.awt.*;
// import java.util.*;

import javax.swing.*;
// import javax.swing.tree.*;

import java.sql.*;

import org.apache.log4j.Logger;

import net.metadiversity.diversity.navigator.*;
import net.metadiversity.diversity.navigator.gui.DnOptionPane;
import net.metadiversity.diversity.navigator.gui.DnMenuItem;
import net.metadiversity.diversity.navigator.gui.FormsProjectEditPanel;
import net.metadiversity.diversity.navigator.connection.*;

/**
 * Describe class <code>ProjectPane</code> here.
 *
 * @author <a href="mailto:didi@china.didi.dn"></a>
 * @version 1.0
 * @since 1.0
 * @see JPanel
 */
public class ProjectPane extends JPanel 
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3256438088703293491L;

    static Logger logger = Logger.getLogger( ConnectionsPane.class.getName() );
    
    final javax.swing.JFrame      mainFrame;
    
    DiversityNavigator      mainWindow;
    ProjectTreeModel        projectTreeModel = null;        
    javax.swing.JTree       tree;        
    javax.swing.JPopupMenu  popRoot;
    javax.swing.JPopupMenu  popProject;
    javax.swing.JPopupMenu  popDBGroup;
    javax.swing.JPopupMenu  popDatabase;
    javax.swing.JPopupMenu  popTable;        
    javax.swing.JScrollPane sp;

    /**
     * Creates a new <code>ProjectPane</code> instance.
     *
     */
    public ProjectPane( final javax.swing.JFrame mainFrame ) 
    {
        super();
        this.mainFrame = mainFrame;
        setLayout( new BorderLayout() );
                                
        projectTreeModel = new ProjectTreeModel( this );                   
        tree             = new JTree( projectTreeModel );
                
        tree.addMouseListener( new java.awt.event.MouseListener() 
        {
            public void mouseClicked( java.awt.event.MouseEvent evt ) {}
            public void mouseEntered( java.awt.event.MouseEvent evt ) {}
            public void mouseExited(  java.awt.event.MouseEvent evt ) {}
            public void mousePressed( java.awt.event.MouseEvent evt ) 
            {
                logger.debug( "mousePressed( " + evt.getModifiers() + " )" );
                // Apple Mac used (ALT) ??? mouse for right butten simulation
                if( ( evt.getModifiers() & java.awt.event.InputEvent.BUTTON3_MASK ) == java.awt.event.InputEvent.BUTTON3_MASK ||
                    ( evt.getModifiers() & java.awt.event.InputEvent.BUTTON2_MASK ) == java.awt.event.InputEvent.BUTTON2_MASK   
                ) 
                {
                    rightClick( evt.getX(), evt.getY() );
                }
            }
            public void mouseReleased( java.awt.event.MouseEvent evt ) {}
        });
                
        sp = new JScrollPane( tree );
        add( sp, BorderLayout.CENTER );
                
        //--------------------------------------------------------------
        // create popup menus
        //--------------------------------------------------------------
        popRoot                         = new JPopupMenu( "Root" );
        popProject                      = new JPopupMenu( "Projects" );
        popDBGroup                      = new JPopupMenu( "DBGroup" );
        popDatabase                     = new JPopupMenu( "Database" );
        popTable                        = new JPopupMenu( "Table" );

        DnMenuItem mnAddProject          = new DnMenuItem( "project.add_project" );
        DnMenuItem mnEditProject         = new DnMenuItem( "project.edit_project" );
        DnMenuItem mnDeleteProject       = new DnMenuItem( "project.delete_project" );
                
        DnMenuItem mnAddDBGroup          = new DnMenuItem( "project.add_group"  );
        DnMenuItem mnEditDBGroup         = new DnMenuItem( "project.edit_group" );
        DnMenuItem mnDeleteDBGroup       = new DnMenuItem( "project.delete_group");
                
        DnMenuItem mnAddDatabase         = new DnMenuItem( "project.add_db" );
        DnMenuItem mnEditDatabase        = new DnMenuItem( "project.edit_db" );
        DnMenuItem mnDeleteDatabase      = new DnMenuItem( "project.delete_db" );
                
        DnMenuItem mnQuery               = new DnMenuItem( "project.open_query_builder" );
        DnMenuItem mnEditRows            = new DnMenuItem( "project.edit_table" );
        DnMenuItem mnOpenProject         = new DnMenuItem( "project.open_project" );

    mnAddProject.addActionListener( new java.awt.event.ActionListener() 
    {
	public void actionPerformed( java.awt.event.ActionEvent evt ) 
        {         
            FormsProjectEditPanel pep = new FormsProjectEditPanel( mainFrame, true, "", "", "" );            
            pep.setVisible( true );                        
        }
    });                
        
    mnEditProject.addActionListener( new java.awt.event.ActionListener() 
    {
	public void actionPerformed( java.awt.event.ActionEvent evt ) 
        {   
            String name = getProjectName();
            
            net.metadiversity.diversity.navigator.project.ProjectInfo pi;        
            pi = net.metadiversity.diversity.navigator.project.ProjectManager.getProjectInfoByName( name );
            
            FormsProjectEditPanel pep = new FormsProjectEditPanel( mainFrame, true, name, pi.getDescription(), pi.getProjectClassName() );            
            pep.setVisible( true );                        
        }
    });                
        
    mnAddDatabase.addActionListener( new java.awt.event.ActionListener() 
    {
	public void actionPerformed( java.awt.event.ActionEvent evt ) 
        { 
            addConnection(); 
        }
    });                
    mnEditDatabase.addActionListener( new java.awt.event.ActionListener() 
    {
	public void actionPerformed( java.awt.event.ActionEvent evt )
        { 
            editConnection(); 
        }
    });
    mnDeleteDatabase.addActionListener( new java.awt.event.ActionListener() 
    {
	public void actionPerformed( java.awt.event.ActionEvent evt ) 
        { 
            deleteConnection(); 
        }
    });
    mnQuery.addActionListener( new java.awt.event.ActionListener() 
    {
	public void actionPerformed( java.awt.event.ActionEvent evt ) 
        { 
            openQuery(); 
        }
    });
    mnEditRows.addActionListener( new java.awt.event.ActionListener() 
    {
	public void actionPerformed( java.awt.event.ActionEvent evt ) 
        { 
            editTable(); 
        }
    });
    mnOpenProject.addActionListener( new java.awt.event.ActionListener() 
    {
	public void actionPerformed( java.awt.event.ActionEvent evt ) 
        {
            // Collection Gui
            openProject();
        }
    });
                
    popRoot.add(    mnAddProject        );
                
    popProject.add( mnAddDBGroup        );
    popProject.add( new JSeparator()    );
    popProject.add( mnOpenProject       );
    popProject.add( new JSeparator()    );
    popProject.add( mnEditProject       );
    popProject.add( mnDeleteProject     );
                
    popDBGroup.add( mnAddDatabase       );
    popDBGroup.add( new JSeparator()    );
    popDBGroup.add( mnEditDBGroup       );
    popDBGroup.add( mnDeleteDBGroup     );
                
    popDatabase.add(    mnQuery             );
    popDatabase.add(    new JSeparator()    );
    popDatabase.add(    mnEditDatabase    );
    popDatabase.add(    mnDeleteDatabase  );
                
    popTable.add(       mnEditRows          );
  }
  //----------------------------------------------------------------------
  //----------------------------------------------------------------------
  /**
   * Describe <code>setMainWindow</code> method here.
   *
   * @param w a <code>DiversityNavigator</code> value
   */
  public void setMainWindow( DiversityNavigator w)
  {
    mainWindow = w;
  }
  /**
   * Describe <code>getMainWindow</code> method here.
   *
   * @return a <code>DiversityNavigator</code> value
   */
  public DiversityNavigator getMainWindow() 
  {
    return mainWindow;
  }
  //----------------------------------------------------------------------
  public String getProjectName() 
  {
    if (tree.getSelectionPath() == null) return null;
    ProjectNode sel = (ProjectNode) tree.getSelectionPath().getLastPathComponent();
    return sel.getProjectName();
  }
  /**
   * Describe <code>getConnectionInfo</code> method here.
   *
   * @return a <code>ConnectionInfo</code> value
   */
  public ConnectionInfo getConnectionInfo() 
  {
    if( tree.getSelectionPath() == null ) 
      return null;
    ProjectNode sel = (ProjectNode) tree.getSelectionPath().getLastPathComponent();
    return sel.getConnectionInfo();
  }
  //----------------------------------------------------------------------
  /**
   * Describe <code>getConnection</code> method here.
   *
   * @return a <code>Connection</code> value
   */
  public Connection getConnection() 
  {
        if( tree.getSelectionPath() == null) 
        {    
            return null;
        }    
        ProjectNode sel = (ProjectNode) tree.getSelectionPath().getLastPathComponent();
        return sel.getConnection();
  }
  //----------------------------------------------------------------------
  /**
   * Describe <code>getTable</code> method here.
   *
   * @return a <code>String</code> value
   */
  public String getTable() 
  {
        if( tree.getSelectionPath() == null ) 
        {
            return null;
        }
        ProjectNode sel = (ProjectNode) tree.getSelectionPath().getLastPathComponent();
        return sel.getTable();
  }
  //----------------------------------------------------------------------
  /**
   * Describe <code>rightClick</code> method here.
   *
   * @param xx an <code>int</code> value
   * @param yy an <code>int</code> value
   */
  private void rightClick(int xx,int yy) 
  {
        if( tree.getSelectionPath() == null ) 
        {
            return;
        }
        int x = xx - sp.getHorizontalScrollBar().getValue();
        int y = yy - sp.getVerticalScrollBar().getValue();
        ProjectNode sel = (ProjectNode) tree.getSelectionPath().getLastPathComponent();
        if( sel == null)
        {    
            return;
        }    
        //--------------------------------------
        // select popup Menus for each Node type
        //--------------------------------------
        switch( sel.getNodeType() )
        {
            case ProjectNode.NODE_TYPE_PROJECTS_CAPTION :
                popRoot.show( this, x,y );
                break;
            case ProjectNode.NODE_TYPE_PROJECT :
                popProject.show( this, x,y );
                break;
            case ProjectNode.NODE_TYPE_DB_GROUP :
                popDBGroup.show( this, x,y );
                break;
            case ProjectNode.NODE_TYPE_DATABASE :
                popDatabase.show( this, x,y );
                break;
            case ProjectNode.NODE_TYPE_TABLE :
                popTable.show( this, x,y );
            break;
        }
    }
    //----------------------------------------------------------------------
    /**
     * Describe <code>addConnection</code> method here.
     *
     */
    public void addConnection() 
    {
/* ???  
        ConnectionDialog conDialog = new ConnectionDialog(null);
        conDialog.setModal( true );
        conDialog.show();
        if( conDialog.getOK() ) 
        {
            try 
            {
                ConnectionsManager.addConnection(conDialog.getName(), conDialog.getDescription(), conDialog.getDriver(), conDialog.getUrl(), conDialog.getUser(), conDialog.getPwd());
            } 
            catch( Exception ex ) 
            {
                logger.error( ex.getMessage(), ex.fillInStackTrace() );
            }
            projectTreeModel.reload();
        }
 */
    }
    public void editConnection() 
    {
/* ???        
        ConnectionInfo ci = getConnectionInfo();
        if( ci == null ) 
        {    
            return;
        }    
        ConnectionDialog conDialog = new ConnectionDialog( ci );
        conDialog.setModal( true );
        conDialog.show();
        if( conDialog.getOK() ) 
        {
            try                         
            {
                ConnectionsManager.changeConnection(conDialog.getName(), conDialog.getDescription(), conDialog.getDriver(), conDialog.getUrl(), conDialog.getUser(), conDialog.getPwd());
            } 
            catch( Exception ex ) 
            {
                logger.error( ex.getMessage(), ex.fillInStackTrace() );
            }
            projectTreeModel.reloadSubTree((ProjectNode) tree.getSelectionPath().getLastPathComponent());
        }
*/    
    }
    //----------------------------------------------------------------------
    /**
     * Describe <code>deleteConnection</code> method here.
     *
     */
    public void deleteConnection() 
    {
        ConnectionInfo ci = getConnectionInfo();
        if( ci == null ) 
        {    
            return;
        }    
        if( DnOptionPane.showConfirmDialog( null, "Do you really want to delete this connection: " + ci.getName(), "Confirmation", DnOptionPane.YES_NO_OPTION ) == DnOptionPane.OK_OPTION )
        {
            try 
            {
                ConnectionsManager.deleteConnection( ci.getName() );
            } 
            catch( Exception ex ) 
            {
                logger.error( ex.getMessage(), ex.fillInStackTrace() );
            }
            projectTreeModel.reloadSubTree((ProjectNode) tree.getSelectionPath().getLastPathComponent());
        }
    }
    //----------------------------------------------------------------------
    /**
     * Describe <code>openQuery</code> method here.
     *
     */
    private void openQuery() 
    {
        ConnectionInfo ci = getConnectionInfo();
        if( ci == null ) 
        {    
            return;
        }    
        Connection con = getConnection();
        if( con == null ) 
        {    
            return;
        }    
        getMainWindow().openQuery( ci, con );
    }
    //----------------------------------------------------------------------
    /**
     * Describe <code>editTable</code> method here.
     *
     */
    private void editTable() 
    {
        String TableName = getTable();
        
        if( TableName == null ) 
        {
            return;
        }    
        ConnectionInfo ci = getConnectionInfo();
        if( ci == null ) 
        {    
            return;
        }    
        Connection con = getConnection();
        if( con == null )
        {
            return;
        }
        getMainWindow().openTable( ci, con, TableName );
    }
    //-----------------------------------------------------------------------
    private void openProject()
    {
        String projectName = getProjectName();
        getMainWindow().openProject( projectName );
    } 
}
