/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * ResultTableModel.java
 *
 * Created on January 17, 2002, 5:20 PM
 */

package net.metadiversity.diversity.navigator.table;
// java
import java.sql.*;
import javax.swing.table.*;
import java.util.*;
// Diversity Navigator
import net.metadiversity.diversity.navigator.*;

import org.apache.log4j.Logger;

/**
 *
 * @author  didi
 */
public class ResultsTableModel extends AbstractTableModel
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3257285837871135026L;

    static Logger logger = Logger.getLogger( ResultsTableModel.class );
    
    private Vector rows;
    private Vector columms;

    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public int getRowCount() 
    {
        logger.debug( "getRowCount()" );
	if( rows == null ) 
        {    
            return 0;
        }    
        else
        {
            return rows.size();
        }    
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public int getColumnCount() 
    {
        logger.debug( "getColumnCount()" );
        
	if( columms == null ) 
        {    
            return 0;
        }
        else
        {
            return columms.size();
        }    
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public String getColumnName(int column ) 
    {
        logger.debug( "getColumnName()" );
        
	return (String)columms.elementAt( column );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public Object getValueAt(int row, int column) 
    {        
        logger.debug( "getValueAt()" );
        
	if( rows == null || columms == null )
        {    
            return null;
        }
        else
        {
            Vector _row = ( Vector ) rows.elementAt( row );
            return _row.elementAt( column );
        }
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void execute( Connection con, String sql, DiversityNavigator mainWindow ) 
    {
	try 
        {
            rows    = new Vector();
            columms = new Vector();

            Statement query = con.createStatement();
            ResultSet result = query.executeQuery( sql );
            ResultSetMetaData rmd = result.getMetaData();
            mainWindow.setQueryStatus( "executing query ..." );
            logger.info( "executing query ..." );
            logger.info( "sql" );
            
            for( int i=1; i <= rmd.getColumnCount(); i++ ) 
            {
		columms.add( rmd.getColumnName( i ) );
            }
            mainWindow.setQueryStatus( "retrieving rows ..." );
            while( result.next() ) 
            {
		Vector row = new Vector();
		for( int i = 1; i <= columms.size(); i++ ) 
                {
                    row.add( result.getString(i) );
		}
		rows.add( row );
            }
            
            result.close();
            query.close();
            
            mainWindow.setQueryStatus( rows.size() + " rows returned" );
            logger.info( rows.size() + " rows returned" );
	} 
        catch( Exception ex ) 
        {		
            // logger.error( "execute(): " + ex.printStackTrace() );
            logger.error( "execute(): " + ex.toString() );
                       
            mainWindow.setQueryError( ex.getMessage() );
	}
	fireTableStructureChanged();
    }
}