/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * TableTableModel.java
 *
 * Created on January 17, 2002, 4:22 PM
 */

package net.metadiversity.diversity.navigator.table;

//import java.awt.*;
//import java.awt.event.*;
// SQL
import java.sql.*;
// Swing GUI
import javax.swing.JTable;
//import javax.swing.JCheckBox;
//import javax.swing.JTextArea;
import javax.swing.table.*;
//import javax.swing.table.TableCellRenderer;
import javax.swing.DefaultCellEditor;

import java.util.*;

import net.metadiversity.diversity.navigator.*;
import net.metadiversity.diversity.navigator.gui.DnOptionPane;
import net.metadiversity.diversity.navigator.types.DnBoolean;
import net.metadiversity.diversity.navigator.types.DnBooleanTableCellRenderer;
import net.metadiversity.diversity.navigator.types.DnBooleanTableCellEditor;

import org.apache.log4j.Logger;

/**
 *
 * @author  didi
 */
public class TableTableModel extends AbstractTableModel
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3256718481298110516L;

    static Logger logger = Logger.getLogger( TableTableModel.class.getName() );
    
    private String          tableName;
    private String          databaseProductName;
    private Vector          rowVector;
    private Vector          columnsVector;
    private Vector          columnsInfo;
    private JTable          inTable;
    private RowChange []    rowChanges;
    private Vector          deleted;
    
    private Vector          importedKeys;
    private Vector          exportedKeys;
    private Vector          foreignKeys;

    final static private String quot = "\"";
    final static boolean debugFlag = false;
        
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void setInTable( JTable inTable ) 
    {               
        if( debugFlag ) logger.debug( "setInTable()" );
        this.inTable = inTable;
        
        // Set up Integer editor
        setUpIntegerEditor( inTable );
        
        // Set up DnBoolean editor
        inTable.setDefaultRenderer( DnBoolean.class, new DnBooleanTableCellRenderer() );
        inTable.setDefaultEditor(   DnBoolean.class, new DnBooleanTableCellEditor() );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private void setUpIntegerEditor( JTable table ) 
    {
        if( debugFlag ) logger.debug( "setUpIntegerEditor()" );
             
        //Set up the editor for the integer cells.
        final WholeNumberField integerField = new WholeNumberField( 0, 5 );
        integerField.setHorizontalAlignment( WholeNumberField.RIGHT );

        DefaultCellEditor integerEditor =  new DefaultCellEditor( integerField ) 
        {
                //Override DefaultCellEditor's getCellEditorValue method
                //to return an Integer, not a String:
                public Object getCellEditorValue() 
                {
                    return new Integer( integerField.getValue() );
                }
        };
        table.setDefaultEditor( Integer.class, integerEditor );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public int getRowCount() 
    {
        // if( debugFlag ) logger.debug( "getRowCount()" );
        if( rowVector == null )
        {    
            return 0;
        }    
        return rowVector.size();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public int getColumnCount() 
    {
        if( debugFlag ) logger.debug( "getColumnCount()" );
        if( columnsVector == null ) 
        {
            return 0;
        }
        return columnsVector.size();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public String getColumnName( int col ) 
    {
        if( debugFlag ) logger.debug( "getColumnName( " + col + " )" );
        return ( String ) columnsVector.elementAt( col );
    }
    //----------------------------------------------------------------------
    //
    //----------------------------------------------------------------------
    public short getColumnClassType( int i )
    {
        if( debugFlag ) logger.debug( "getColumnClassType( " + i + " )" );            
        ColumnInfo ci = ( ColumnInfo ) columnsInfo.elementAt( i );
        return ci.getDataType();            
    }
    //----------------------------------------------------------------------
    //
    //----------------------------------------------------------------------
    public Class getColumnClass( int i )
    {            
        if( debugFlag ) logger.debug( "getColumnClass( " + i + " )" );

        ColumnInfo ci = ( ColumnInfo ) columnsInfo.elementAt( i );

         switch( ci.getDataType() )
         { 
            // use Checkbox
            case java.sql.Types.BIT:     
                    logger.debug( "Type is Boolean" );
                    return ci.isClassOf();
            // use Integer editor        
            case java.sql.Types.INTEGER :        
            case java.sql.Types.TINYINT :
                /* ??? can't insert 'null' value
                    return ci.isClassOf();
                 */
            case java.sql.Types.ARRAY :        
            case java.sql.Types.BIGINT :        
            case java.sql.Types.BINARY :        
            case java.sql.Types.BLOB :        
            case java.sql.Types.CHAR :        
            case java.sql.Types.CLOB :        
            case java.sql.Types.DATE :        
            case java.sql.Types.DECIMAL :        
            case java.sql.Types.DISTINCT :        
            case java.sql.Types.DOUBLE :        
            case java.sql.Types.FLOAT :        
            case java.sql.Types.JAVA_OBJECT :        
            case java.sql.Types.LONGVARBINARY :        
            case java.sql.Types.LONGVARCHAR :        
            case java.sql.Types.NULL :        
            case java.sql.Types.NUMERIC :        
            case java.sql.Types.OTHER :        
            case java.sql.Types.REAL :        
            case java.sql.Types.REF :        
            case java.sql.Types.SMALLINT :        
            case java.sql.Types.STRUCT :        
            case java.sql.Types.TIME :        
            case java.sql.Types.TIMESTAMP :        
            case java.sql.Types.VARBINARY :        
            case java.sql.Types.VARCHAR :        

                    return new String().getClass();
            default:
                    return new String().getClass();                     
         }
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public Object getValueAt( int row, int column ) 
    {
        if( debugFlag ) logger.debug( "getValueAt( row = " + row + ", column = " + column + " )" );

        if( rowVector == null || columnsVector == null ) 
        {    
            return null;
        }
        Vector Row = (Vector) rowVector.elementAt( row );
        return Row.elementAt( column );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private ColumnInfo findColumnInfo( String colName ) 
    {
        if( debugFlag ) logger.debug( "findColumnInfo( colName = " + colName + " )" );
        
	for( Enumeration Enum = columnsInfo.elements(); Enum.hasMoreElements(); ) 
        {
            ColumnInfo ci = (ColumnInfo) Enum.nextElement();
            
            if( ci.getName().equals( colName ) )
            {    
                return ci;
            }
	}
	return null;
    }
    //----------------------------------------------------------------------
    // Get PrimaryKey
    //----------------------------------------------------------------------
    public Vector getPrimaryKey() 
    {
        if( debugFlag ) logger.debug( "getPimaryKey()" );
        Vector ret = new Vector();
        for( Enumeration Enum = columnsInfo.elements(); Enum.hasMoreElements(); ) 
        {
            ColumnInfo ci = (ColumnInfo)Enum.nextElement();
            if( ci.isPrimaryKey() )  
            {
                ret.add( ci );
            }
        }
        return ret;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public TableTableModel( Connection con, String tableName, DiversityNavigator mainWindow ) 
    {
        if( debugFlag ) logger.debug( "TableTableModel( Connection con, String tableName, DiversityNavigator mainWindow )" ); 
        try 
        {
            rowVector       = new Vector();
            columnsVector   = new Vector();
            columnsInfo     = new Vector();

            this.tableName = tableName;                        
            //------------------------------------------------------
            //------------------------------------------------------ 

            DatabaseMetaData dbmd = con.getMetaData();
            ResultSet columnResultSet;

            columnResultSet = dbmd.getColumns( null, null, tableName, null );
            databaseProductName = dbmd.getDatabaseProductName();
            
            while( columnResultSet.next() ) 
            {
                String name         = columnResultSet.getString(    "COLUMN_NAME" );
                short dataType      = columnResultSet.getShort(     "DATA_TYPE"   );
                String typeName     = columnResultSet.getString(    "TYPE_NAME"   );
                int size            = columnResultSet.getInt(       "COLUMN_SIZE" );
                int decimals        = columnResultSet.getInt(       "DECIMAL_DIGITS");
                int nullable        = columnResultSet.getInt(       "NULLABLE"    );
                String defaultVal   = columnResultSet.getString(    "COLUMN_DEF"  );
                boolean nulls       = ( nullable == DatabaseMetaData.columnNullable );

                columnsInfo.add( new ColumnInfo( name, dataType, typeName, size, decimals, nulls, defaultVal ) );
            }
            columnResultSet.close();
            //------------------------------------------------------
            // Get Primary Keys 
            //------------------------------------------------------                        
            ResultSet primaryKeyResultSet = dbmd.getPrimaryKeys( null, null, tableName );
            while( primaryKeyResultSet.next() ) 
            {
                String name = primaryKeyResultSet.getString( "COLUMN_NAME" );
                ColumnInfo ci = findColumnInfo( name );
                ci.setPrimaryKey( true );
            }
            primaryKeyResultSet.close();
            //------------------------------------------------------
            // Get Foreign Keys 
            //------------------------------------------------------                        
            ResultSet importedKeyResultSet = dbmd.getImportedKeys( null, null, tableName );
            foreignKeys  = new Vector();
            importedKeys = new Vector();
            while( importedKeyResultSet.next() ) 
            {                
                StringBuffer strBuf = new StringBuffer();
                           
                strBuf.append("Imported : " );
                strBuf.append( importedKeyResultSet.getString( "FKTABLE_NAME" ) );
                strBuf.append( "." );
                strBuf.append( importedKeyResultSet.getString( "FKCOLUMN_NAME" ) );
                strBuf.append( "." );
                strBuf.append( importedKeyResultSet.getString( "FK_NAME" ) );
                
                String str = strBuf.toString();
                foreignKeys.add( str );
                importedKeys.add( str );
                logger.info( str );
                System.out.println( str );
            }
            importedKeyResultSet.close();
            //------------------------------------------------------
            ResultSet exportedKeyResultSet = dbmd.getExportedKeys( null, null, tableName );
            exportedKeys = new Vector();
            while( exportedKeyResultSet.next() ) 
            {
                StringBuffer strBuf = new StringBuffer();
                           
                strBuf.append("Exported : " );
                strBuf.append( exportedKeyResultSet.getString( "FKTABLE_NAME" ) );
                strBuf.append( "." );
                strBuf.append( exportedKeyResultSet.getString( "FKCOLUMN_NAME" ) );
                strBuf.append( "." );
                strBuf.append( exportedKeyResultSet.getString( "FK_NAME" ) );
                
                String str = strBuf.toString();
                
                foreignKeys.add( str );
                exportedKeys.add( str );           
                logger.info( str );
                System.out.println( str );
            }
            exportedKeyResultSet.close();
            //------------------------------------------------------

            Statement query = con.createStatement();
            mainWindow.setQueryStatus( "executing query ..." );
            ResultSet result = query.executeQuery( "SELECT * FROM \"" + tableName + "\"" );
            ResultSetMetaData rmd = result.getMetaData();

            for( int i = 1; i <= rmd.getColumnCount(); i++ ) 
            {
                 columnsVector.add( rmd.getColumnName( i ) );
            }
            mainWindow.setQueryStatus( "retrieving rows ..." );
            // add data to table, look for the typ of each columne
            while( result.next() ) 
            {
                Vector row = new Vector();
                for( int i = 1; i <= columnsVector.size(); i++ ) 
                {
                    ColumnInfo ci = ( ColumnInfo ) columnsInfo.elementAt( i-1 );
                    switch( ci.getDataType() )
                    {
                        case java.sql.Types.BIT:                                            
                             row.add( new net.metadiversity.diversity.navigator.types.DnBoolean( result.getString( i ) ) );
                              break;
                         /* ??? can't insert 'null' value
                        case java.sql.Types.TINYINT :        
                        case java.sql.Types.INTEGER :        
                             row.add( new Integer( result.getInt( i ) ) );
                             break;
                        */
                        /*    
                        case java.sql.Types.TIMESTAMP:
                              Row.add( R.getTimestamp( i ) );
                              break;
                        */
                        default:
                              row.add( result.getString( i ) );
                              break;
                    }    
                }
                rowVector.add( row );
            }
            result.close();
            query.close();
            //------------------------------------------------------
            //------------------------------------------------------                        

            // System.out.println( "rowVector: " + rowVector.size() );                        
            rowChanges = new RowChange[ rowVector.size() ];

            mainWindow.setQueryStatus( rowVector.size() + " rows returned" );
        } 
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage() );
            mainWindow.setQueryError( ex.getMessage() );
        }
        fireTableStructureChanged();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public int getTablesDetailsCount() 
    {
        return columnsInfo.size();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public Object getTableColumnDetails( int row, int col) 
    {
        if( debugFlag ) logger.debug( "getTableDetailsColumn()" );
        
        if (rowVector == null || columnsVector == null || columnsInfo == null) 
        {
            return "Error";
        }    
        else
        {
            ColumnInfo ci = (ColumnInfo) columnsInfo.get( row );
            
            switch( col )
            {
                case  0 :
                    return ci.getName();
                case  1 :
                    String name = ci.getTypeName();
                    if( name.equalsIgnoreCase( "varchar" ) )
                    {
                        name = name + "( " + ci.getSize() + " )";
                    }    
                    return name;
                case  2 :
                    return Boolean.valueOf( ! ci.isNull() );
                case  3 :
                    return ci.getDefaultValue();
                default :    
                    return "";
            }
        }
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public String getTablesDetails() 
    {
        if( debugFlag ) logger.debug( "getTablesDetailsCount()" );

        if (rowVector == null || columnsVector == null || columnsInfo == null) 
        {
            return "Error";
        }    
        StringBuffer b = new StringBuffer( "" );
        b.append( "columnsVector:\n" );
        for( Enumeration Enum = columnsInfo.elements(); Enum.hasMoreElements(); ) 
        {
                ColumnInfo ci = (ColumnInfo)Enum.nextElement();
                b.append( "\t" + ci.getName() + ": " + ci.getTypeName() + " SQLType=" + ci.getDataType() + " Length=" + ci.getSize() + ( ci.isNull() ? " true ":" false ") + "\n" );
        }
        b.append( "Primary Key:\n\t(" );
        boolean first=true;

        for( Enumeration Enum = getPrimaryKey().elements(); Enum.hasMoreElements(); ) 
        {
            ColumnInfo ci = (ColumnInfo) Enum.nextElement();
            if( first )  
            {
                first = false;
            }  
            else 
            {
                b.append(", ");
            }
            b.append( ci.getName() );
        }
        b.append( ")\n" );
        return b.toString();
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public boolean isCellEditable( int row, int column ) 
    {
        if( debugFlag ) logger.debug( "isCellEditable( " + row + ", " + column + " )" );
        try
        {
            ColumnInfo ci = ( ColumnInfo ) columnsInfo.elementAt( column );
            RowChange rc = rowChanges[ row ];

            if( rc == null || rc.ChangeType == RowChange.CHANGED_COLUMN ) 
            {
                return ! ci.isPrimaryKey();
            }
            return true;
        }
        catch( java.lang.ArrayIndexOutOfBoundsException ex )
        {
                // didi Error !!!
                logger.error( ex.getMessage() );
                return true;
        }
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void setValueAt( Object value, int row, int column ) 
    {
        if( debugFlag ) logger.debug( "setValueAt( " + row + ", " + column + " )" );

        RowChange rc = null;
        try
        {
            rc = rowChanges[ row ];
        }
        catch( java.lang.ArrayIndexOutOfBoundsException e )
        {
            rc = null;
        }
        if( rc == null ) 
        {
                rowChanges[ row ] = new RowChange( RowChange.CHANGED_COLUMN );
        }    
        Vector V = (Vector) rowVector.elementAt( row );
        V.remove( column );
        V.add( column, value );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private String SQLValue( ColumnInfo ci, int row, int col ) 
    {
        String value;
        if( debugFlag ) logger.debug( "SQLValue( " + row + ", " + col +" )" ); 

        Object obj = ( (Vector) (rowVector.elementAt(row))).elementAt(col);

        if( obj == null )
        {
            return SQLValue( ci, null );
        }
        switch( ci.getDataType() )
        {
            case java.sql.Types.BIT :
                value = obj.toString();
                break;
                /*
            case java.sql.Types.DATE :
            case java.sql.Types.TIME :
            case java.sql.Types.TIMESTAMP :
                value = obj.toString();
                if( value.compareToIgnoreCase( "null" ) == 0 )
                    return null;
                else
                    return "'" + value + "'";
                 */
            default:                    
                value = obj.toString();
                break;
        }       
        logger.debug(  "return: " + value ); 
        return SQLValue( ci, value );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    private String SQLValue( ColumnInfo ci, String value ) 
    {
        if( debugFlag ) logger.debug( "SQLValue( ColumnInfo ci, String value )" );
        if( value != null )
        {
            value = value.trim();
            // qute all " to \"
            // System.out.println( value  );
            if( value.indexOf( '"' ) >= 0 || value.indexOf( '\'' ) >= 0 )
            {
                StringBuffer ret = new StringBuffer();;                    
                for( int i = 0; i < value.length(); i++ )
                {
                    char c = value.charAt( i );
                    switch( c )
                    {
                        case '\"' :
                            ret.append( "\\\"" );
                            break;
                        case '\'' :
                            ret.append( "\\\'" );
                            break;
                        default :
                            ret.append( c );
                    }
                }
                value = ret.toString();
            }    
            // System.out.println( value );
        }
        if( value == null && ci.isNull() )
        {    
            return "null";
        }    
        if( ci.isNull() && value.length() == 0 ) 
        {    
            return "null";
        }
        switch( ci.getDataType() ) 
        {
            case java.sql.Types.BIGINT :
            case java.sql.Types.DECIMAL :
            case java.sql.Types.DOUBLE :
            case java.sql.Types.FLOAT :
            case java.sql.Types.INTEGER :
            case java.sql.Types.NUMERIC :
            case java.sql.Types.REAL :
            case java.sql.Types.SMALLINT :
            case java.sql.Types.TINYINT :
                
                if( value.compareToIgnoreCase( "null" ) == 0 )
                {
                    return "null";
                }
                else if( value.compareToIgnoreCase( "" ) == 0  )
                {                    
                    return ci.getDefaultValue();
                }
                value = value.replace( ',', '.' );
                return "'" + value + "'";
            case java.sql.Types.BIT :                      
                // logger.debug( "Bit value : " + value );
                return value;           // don't use quotes
            case java.sql.Types.DATE :
            case java.sql.Types.TIME :
            case java.sql.Types.TIMESTAMP :
                if( value.compareToIgnoreCase( "null" ) == 0 )
                {
                    return "null";
                }
                else if( value.compareToIgnoreCase( "" ) == 0  )
                {                    
                    return ci.getDefaultValue();
                }
                else
                {    
                    return "'" + value + "'";
                }
            default:
                return "'" + value + "'";
        }
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void save( Connection con ) 
    {            
        boolean logErrorsOnly = true;
        if( debugFlag ) logger.debug( "save" );
        if( ! logErrorsOnly )
        {
            logger.info( "Sending queries to server" );
        }

        for( int i=0; i < rowVector.size(); i++ ) 
        {
            RowChange rc = rowChanges[ i ];

            if( rc != null ) 
            {
                Vector Row = (Vector)   rowVector.elementAt( i );
                StringBuffer sql = new StringBuffer();
                switch( rc.ChangeType ) 
                {
                //----------------------------
                // Insert into database    
                //----------------------------    
                case RowChange.NEW_COLUMN:

                    logger.debug( "insert:" );

                    
                    if( databaseProductName.compareTo( "PostgreSQL") == 0 )
                    {
                        // set timeformat
                        sql.append( "SET DATESTYLE TO EUROPEAN, GERMAN;\n" );                         
                    }
                    sql.append( "INSERT INTO " + quot + tableName + quot +" (" );
                    String ValuesPart="";
                    boolean first=true;
                    int columnCount = columnsInfo.size();
                    for( int c = 0; c < columnCount; c++ ) 
                    {
                        ColumnInfo ci = ( ColumnInfo ) columnsInfo.elementAt( c );
                        if( first ) 
                        {    
                            sql.append( quot );
                            first   =  false;
                        }    
                        else 
                        {
                            sql.append( quot + ", " + quot );
                            ValuesPart += ", ";
                        }
                        sql.append( ci.getName() );
                        ValuesPart += SQLValue( ci, i, c );
                    }
                    sql.append( quot + ") \n  VALUES \n  (" + ValuesPart + ")\n" );
                    break;				
                //----------------------------    
                // Update row
                //----------------------------    
                case RowChange.CHANGED_COLUMN:

                    logger.debug( "update:" );
                    sql.append( "UPDATE " + quot + tableName + quot );
                    String SETPart      = " SET ";
                    String WHEREPart    = " WHERE ";
                    boolean firstField  = true;
                    boolean firstKey    = true;

                    for( int c = 0; c < columnsInfo.size(); c++ ) 
                    {
                        ColumnInfo ci = ( ColumnInfo ) columnsInfo.elementAt( c );
                        if( ci.isPrimaryKey() )
                        {
                            if( firstKey ) 
                            {
                                firstKey = false;
                            }
                            else 
                            {
                                WHEREPart += "\n   AND ";
                            }    
                            WHEREPart +=  quot + ci.getName() + quot + " = " + SQLValue( ci, i, c );
                        } 
                        else 
                        {
                            if( firstField ) 
                            {
                                firstField = false;
                            }
                            else 
                            {
                                SETPart += "\n  , ";
                            }    
                            SETPart += quot + ci.getName() + quot + " = " + SQLValue( ci, i, c );
                        }
                    }
                    sql.append( "\n" + SETPart + "\n" + WHEREPart ); 
                    break;				
                }
                if( debugFlag ) logger.debug( sql );
                
                Statement query = null;
                try 
                {
                    query = con.createStatement();
                    int r = query.executeUpdate( sql.toString() );
                    rowChanges[ i ] = null;
                    if( ! logErrorsOnly )
                    {
                        logger.info( "Execute Statment ... " + sql.toString() + "\n" + r + " rows affected\n\n" );
                    }    
                } 
                catch( SQLException sql_ex ) 
                {
                   logger.error( "SQL Error: " + sql_ex.getMessage() + "\n" +  sql.toString() );
                   if( DnOptionPane.showConfirmDialog( null, sql_ex.getMessage(), "SQL Error", DnOptionPane.OK_CANCEL_OPTION ) == DnOptionPane.CANCEL_OPTION )
                   {
                       logger.error( "aborted by user" );
                       return;
                   }
                } 
                catch( Exception ex ) 
                {
                    logger.fatal( "Application Error: " + ex.getMessage() );
                }
                try 
                {
                    query.close();
                } 
                catch( Exception ex ) 
                {
                    ex.printStackTrace();
                }
            }
        }
        //--------------------------------------------------------------
        // deleted row
        //--------------------------------------------------------------
        if( deleted != null ) 
        {
            if( debugFlag ) logger.debug( "delete:" );

            Enumeration e = deleted.elements();
            while( e.hasMoreElements()  ) 
            {
                Object test = e.nextElement();

                Vector row = ((Vector) test );
                StringBuffer sql = new StringBuffer();
                sql.append( "DELETE FROM  " + quot + tableName + quot );
                String WHEREPart = " WHERE ";
                boolean firstKey = true;
                for( int c = 0; c < columnsInfo.size(); c++ ) 
                {
                    ColumnInfo ci = ( ColumnInfo ) columnsInfo.elementAt( c );
                    if( ci.isPrimaryKey() ) 
                    {
                        if( firstKey ) 
                        {
                            firstKey = false;
                        }    
                        else 
                        {
                            WHEREPart += "\n  AND ";
                        }    
                        WHEREPart += quot + ci.getName() + quot + " = " + SQLValue( ci, row.elementAt( c ).toString() );
                    }
                }
                sql.append( "\n" + WHEREPart );
                Statement query = null;
                try 
                {
                    query = con.createStatement();
                    int r = query.executeUpdate( sql.toString() );
                    if( ! logErrorsOnly )
                    {
                        logger.info( "Execute Statment ... " + sql.toString() + "\n" + r + " rows affected\n\n" + r + " rows affected" );
                    }    
                    
                } 
                catch( SQLException sql_ex ) 
                {
                   logger.error( "SQL Error: " + sql_ex.getMessage() + "\n" +  sql.toString() );
                   if( DnOptionPane.showConfirmDialog( null, sql_ex.getMessage(), "SQL Error", DnOptionPane.OK_CANCEL_OPTION ) == DnOptionPane.OK_OPTION )
                   {
                       return;
                   }
                } 
                catch( Exception ex ) 
                {
                    logger.error( "Application Error: " + ex.getMessage() );
                }
                try 
                {
                    query.close();
                } 
                catch( Exception ex ) 
                {
                    ex.printStackTrace();
                }
            }
            deleted = null;
        }
    }
    //----------------------------------------------------------------------
    // create a new row.
    //----------------------------------------------------------------------
    public void newRow() 
    {
        if( debugFlag ) logger.debug( "newRow()" );
        Vector row = new Vector();
        for( int i = 0; i < columnsVector.size(); i++ ) 
        {
            short type = getColumnClassType( i );                
            switch( type )
            {
                        case java.sql.Types.BIT :        
                                // logger.debug( "Type is BIT" );
                                row.add( new net.metadiversity.diversity.navigator.types.DnBoolean( 0 ) );
                                break;
                        case java.sql.Types.INTEGER :        
                        case java.sql.Types.TINYINT :        
                                // logger.debug( "Type is INT" );
                                /* ??? can't insert 'null' value
                                row.add( new Integer( 0 ) );
                                break;
                                 */
                                row.add( "" );
                                break;
                        case java.sql.Types.DATE :
                        case java.sql.Types.TIME :
                        case java.sql.Types.TIMESTAMP :
                                row.add( "" );
                                break;                                    
                        default :
                                // logger.debug( "Type is STRING" );
                                row.add( "" );                                        
                                break;
            }                                                    
        }    
        rowVector.add( row );
        RowChange[] R = new RowChange[ rowVector.size() ];
        System.arraycopy( rowChanges, 0, R, 0, rowChanges.length );
        R[ rowVector.size() - 1 ] = new RowChange( RowChange.NEW_COLUMN );
        rowChanges = R;
        fireTableRowsInserted( rowVector.size(), rowVector.size() );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void addRow( Vector row ) 
    {
        if( debugFlag ) logger.debug( "addRow()" );
        rowVector.add( row );
        RowChange[] R = new RowChange[ rowVector.size() ];
        System.arraycopy( rowChanges, 0, R, 0, rowChanges.length );
        R[ rowVector.size() - 1 ] = new RowChange( RowChange.NEW_COLUMN );
        rowChanges = R;
        fireTableRowsInserted( rowVector.size(), rowVector.size() );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public void deleteRow( int row ) 
    {
        if( debugFlag ) logger.debug( "deleteRow()" );
        RowChange rch = rowChanges[ row ];
        if( rch != null ) 
        {
            if( rch.ChangeType == RowChange.NEW_COLUMN ) 
            {
                rowChanges[ row ] = null;
                fireTableRowsDeleted( row, row );
                rowVector.remove( row );
                return;
            }
        }
        Vector RemR = (Vector) rowVector.elementAt( row );
        if( deleted == null )
        {    
            deleted = new Vector();
        }    
        deleted.add( RemR );
        RowChange[] R = new RowChange[rowVector.size() - 1];
        System.arraycopy( rowChanges, 0, R, 0, row );
        System.arraycopy( rowChanges, row + 1, R, row, rowChanges.length - row - 1 );
        rowVector.remove( row );
        rowChanges = R;
        fireTableRowsDeleted( row, row );
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public Vector getImportedKeys()
    {
        return importedKeys;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public Vector getExportedKeys()
    {
        return exportedKeys;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public Vector getForeignKeys()
    {
        return foreignKeys;
    }
    //----------------------------------------------------------------------
    //----------------------------------------------------------------------
    public String getTableName()
    {
        return tableName;
    }
}
