/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * DnBooleanEditor.java
 *
 * Created on 10. August 2002, 15:19
 *
 * @author  didi
 */

package net.metadiversity.diversity.navigator.types;

import java.awt.*;
import java.awt.Component;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;

import net.metadiversity.diversity.navigator.*;     // icons
import org.apache.log4j.Logger;

public class DnBooleanTableCellEditor extends AbstractCellEditor implements TableCellEditor
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3904681561449247283L;

    static Logger logger = Logger.getLogger( DnBooleanTableCellEditor.class.getName() );
    
    private int    val;
    private int    r,c; 
    private JLabel label;

    static boolean debugFlag = false;
    
    public DnBooleanTableCellEditor()
    {
        logger.debug( "DnBooleanTableCellEditor()" );
        label = new JLabel();
        label.setMaximumSize( new Dimension( 16, 50 ) );
    }

    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column)
    {
        logger.debug( "getTableCellEditorComponent( row : " + row + " col : " + column + " )" );
        logger.debug( "isSelected : " + isSelected );

        // ??? val = ((DnBoolean) value).getValue();
        
        int val = 0;
        try
        {
            val = ((DnBoolean) value).getValue();    
        }
        catch( ClassCastException ex )
        {
            logger.error( ex.getLocalizedMessage() );
        }

        
        r = row; c = column;
        if( debugFlag ) logger.debug( "( " + r + "," + c + " ) val = " + val );
//        val++;
//        if( val >= 2 ) val = -1;
        switch( val )
        {
           case -1 :
                    if( debugFlag ) logger.debug( "set false" );
                    label.setText( "false" );
                    label.setIcon( DiversityNavigatorProperties.getCheckboxFalseIcon() );
                    ((DnBoolean) value).setValue( DnBoolean.valFalse );
                    break;
           case 0  :
                    if( debugFlag )  logger.debug( "set null" );
                    label.setText( "null" );
                    label.setIcon( DiversityNavigatorProperties.getCheckboxNullIcon() );
                    ((DnBoolean) value).setValue( DnBoolean.valNull );
                    break;
           case 1  :
                    if( debugFlag ) logger.debug( "set true" );
                    label.setText( "true" );
                    label.setIcon( DiversityNavigatorProperties.getCheckboxTrueIcon() );
                    ((DnBoolean) value).setValue( DnBoolean.valTrue );
                    break;
        }
        return label;
    }

    public boolean shouldSelectCell( EventObject anEvent )
    {
        val++;
        if( val >= 2 ) val = -1;
        if( debugFlag )  logger.debug( "shouldSelectCell()" );
        if( debugFlag ) logger.debug( "( " + r + "," + c + " ) val = " + val );
        stopCellEditing();
        return true;
    }

    public void cancelCellEditing()
    {
        if( debugFlag ) logger.debug( "cancelCellEditing()" );
        if( debugFlag ) logger.debug( "( " + r + "," + c + " ) val = " + val );
        // Bearbeitung abgebrochen - Dialogfeld ausblenden
        // colorDialog.setVisible(false);
        super.cancelCellEditing();
    }

    public boolean stopCellEditing()
    {
        if( debugFlag ) logger.debug( "stopCellEditing()" );
        if( debugFlag ) logger.debug( "( " + r + "," + c + " ) val = " + val );
        
        switch( val )
        {
           case -1 :
                    if( debugFlag ) logger.debug( "set false" );
                    label.setText( "false" );
                    label.setIcon( DiversityNavigatorProperties.getCheckboxFalseIcon() );
                    break;
           case 0  :
                    if( debugFlag ) logger.debug( "set null" );
                    label.setText( "null" );
                    label.setIcon( DiversityNavigatorProperties.getCheckboxNullIcon() );
                    break;
           case 1  :
                    if( debugFlag ) logger.debug( "set true" );
                    label.setText( "true" );
                    label.setIcon( DiversityNavigatorProperties.getCheckboxTrueIcon() );
                    break;
        }
        label.setVisible( true );
        super.stopCellEditing();
        return true;
    }

    public Object getCellEditorValue()
    {
        if( debugFlag ) logger.debug( "getCellEditorValue()" );
        if( debugFlag ) logger.debug( "( " + r + "," + c + " ) val = " + val );
        return new DnBoolean( val );
    }
}
