/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * LogViewer.java
 *
 * Created on 30. April 2003, 17:01
 */

package net.metadiversity.diversity.navigator.util;

import java.io.*;

import java.util.ArrayList;

import javax.swing.*;
import java.awt.*;

// import java.awt.event.*;
// import javax.swing.text.StyledDocument;
// import javax.swing.text.SimpleAttributeSet;
// import javax.swing.text.MutableAttributeSet;
// import javax.swing.text.StyleConstants;

import org.apache.log4j.*;
import org.apache.log4j.spi.ThrowableInformation;
import org.apache.log4j.spi.LoggingEvent;

// import org.apache.log4j.WriterAppender;
// import org.apache.log4j.helpers.Loader;
// import org.apache.log4j.helpers.OptionConverter;

/**
 *
 * @author  didi
 */
public class LogViewer extends org.apache.log4j.WriterAppender 
{
    private String          loggerName = "DN-GUI";
    private Logger          logger;
    private StringWriter    sw;
    private LogListPanel    logListPanel;

    private ArrayList       events;
    private Level           triggerLevel;
    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    
    /** Creates a new instance of LogViewer */
    public LogViewer() 
    {
        super();
        // setup the logging
        sw = new StringWriter();
//        this.setLayout( new PatternLayout( "%-5p %d [%t]:  %m%n" ) );
        this.setLayout( new PatternLayout( "%-8p [%t]:  %m%n" ) );
        this.setWriter( sw );
        this.setThreshold( Priority.DEBUG );
        this.setImmediateFlush( true );
        
        // System.out.println( "GUI logger init: " + this.checkEntryConditions() );
        
//        logger = Logger.getLogger( loggerName );
        logger = Logger.getRootLogger();
        logger.addAppender( this );                
        
        events = new ArrayList( 1000 );
        triggerLevel = Level.DEBUG;
        
        // GUI        
        setLogListPanel( new LogListPanel() );    
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void append( LoggingEvent event ) 
    {
        // System.out.print( "append --> " ); 
        String text = this.layout.format( event );        
       
        // publishEvent( event.getLevel(), text );
        if( event.getMessage() != null )
        {    
            publishEvent( event.getLevel(), text );
        }    
        ThrowableInformation t  = event.getThrowableInformation();        
        // print stack trace        
        if( t != null )
        {   
            Throwable th = t.getThrowable();
            if( th != null )
            {    
                java.lang.StackTraceElement ste [] = th.getStackTrace();            
                for( int j = 0; j < ste.length; j++ )
                {    
                    // System.out.println( "stack -->" + ste[ j ].toString()  );                
                    publishEvent( event.getLevel(), "        " + ste[ j ].toString() );
                }    
            }
        }     
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------    
    public void test()
    {
        try
        {
             int i = 10 / 0;
        }                
        catch( java.lang.Exception ex )
        {
             // System.out.println( ex.getCause() );
             // System.out.println( ex.getMessage() );
            
             logger.debug( "--- logger debug level -----------------------------------" );
             logger.info(  "--- logger info level  -----------------------------------" );
             logger.warn(  "--- logger warn level  -----------------------------------" );
             logger.error( "--- logger error level -----------------------------------" );
             logger.fatal( "--- logger fatal level -----------------------------------" );
             logger.error( ex.getMessage(), ex.fillInStackTrace() );
       }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void testLogPanel() 
    {
        JFrame logFrame = new JFrame( "TestLog" );        
        logFrame.getContentPane().add( getLogListPanel() );
        logFrame.validate();
        logFrame.pack();
        logFrame.setVisible( true );
        
        logFrame.addWindowListener(new java.awt.event.WindowAdapter()
        {
            public void windowClosing(java.awt.event.WindowEvent evt)
            {
               System.exit( 0 );
            }
        });

    }    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public JPanel getLogListPanel() 
    {
        return logListPanel;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public  void setName(String name) 
    {
        this.name = name;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    protected void setLogListPanel( LogListPanel logListPanel ) 
    {
        this.logListPanel = logListPanel;
        logListPanel.setTextBackground( Color.white );
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void publishEvent( Level level, String text ) 
    {
        // System.out.println( "publishEvent" );
        synchronized( events ) 
        {
            LogEvent event = new LogEvent( level, text );
            events.add( event );
            logListPanel.newEvent( event );
            if( triggerLevel != null && triggerLevel.isGreaterOrEqual( level ) )
            {   
                events.notify();
            }
        }
    }
    /**
     * @param args the command line arguments
     */
    public static void main( String[] args ) 
    {
        LogViewer lv = new LogViewer();
        System.out.println( "init finish" );
        lv.testLogPanel();
        int loop = 0;
        while( loop++ < 100 )
        {
            lv.test();                        
        }
    }    
}
